@@ -1,3 +1,30 @@
+/* -*- mode: C; mode: fold; -*- */
+/* This file is a MESS.  For that reason, I wrote sltcp.c which I regard as 
+ * much cleaner.  I tried to clean this up but with at least 4 different
+ * TCP/IP implementations for VMS as well as TLI and BSD, there is not too
+ * much hope.  The routines in sltcp.c should be adequate for most Unix 
+ * systems as well as OS/2.  Until VMS standardizes its TCP/IP interface
+ * the routines in this file will have to be used.
+ * 
+ * Much of this file has been totally re-written.  Now all read/writes are 
+ * to a file descriptor.  All reads take place via the function 
+ * 'client_read' and all writes are done in 'client_write'.  These functions
+ * call the appropriate low level functions to perform the actual operation.
+ * Specifically, client_read calls:
+ *  
+ *     socket_read (MULTINET)
+ *     do_netlib_read  (NETLIB)   [Note: this function needs to be created.]
+ *     read   (everything else)
+ * 
+ * and client_write calls:
+ * 
+ *     netlib_write (NETLIB)
+ *     socket_write (MULTINET)
+ *     write  (everything else)
+ * 
+ * JED.
+ */
+
 /*
  * This software is Copyright 1991 by Stan Barber.
  *
@@ -13,26 +40,18 @@
  *
  */
 
-/* Modified 'server_init' by John E. Davis for use by slrn. */
-
-/*
- * NNTP client routines.
- */
-
-/*
- * Include configuration parameters only if we're made in the nntp tree.
- */
 
 #if 0
 # define DEBUG_FILE "slrn.log" 
 #endif
 
 #include "config.h"
-#include "features.h"
-
+#include "slrnfeat.h"
 
 #include <stdio.h>
 #include <string.h>
+#include <signal.h>
+#include <errno.h>
 
 #ifdef HAVE_UNISTD_H
 # include <unistd.h>
@@ -42,8 +61,11 @@
 # include <stdlib.h>
 #endif
 
+#include <sys/types.h>
+
 #ifdef VMS
-# include <signal.h>
+/*{{{ VMS includes */
+# include "vms.h"
 # ifdef MULTINET
 #  include "multinet_root:[multinet.include.vms]inetiodef.h"
 #  include "multinet_root:[multinet.include.sys]types.h"
@@ -64,14 +86,14 @@
 #   define	NNTP_PORT 119
 #  endif
 # endif
-#else /* !VMS */
-# include <sys/types.h>
+/*}}}*/
 #endif
 
 #include <ctype.h>
 
 
 #ifdef TLI
+/*{{{ TLI includes */
 # include	<fcntl.h>
 # include	<tiuser.h>
 # include	<stropts.h>
@@ -85,8 +107,10 @@
 # endif
 # define	IPPORT_NNTP	((unsigned short) 119)
 # include 	<netdb.h>	/* All TLI implementations may not have this */
+/*}}}*/
 #else /* !TLI */
 # ifdef VMS
+/*{{{ More VMS includes */
 #  ifdef MULTINET
 #   include "multinet_root:[multinet.include]netdb.h"
 #   include "multinet_root:[multinet.include.sys]socket.h"
@@ -102,7 +126,9 @@
 #   include <netdb.h>
 #   include <socket.h>
 #   include <in.h>
+#   include <inet.h>
 #  endif
+/*}}}*/
 # else /* !VMS */
 #  include <sys/socket.h>
 #  include <netinet/in.h>
@@ -115,8 +141,6 @@
 #endif /* !TLI */
  
 
-#include <sys/types.h>
-
 #ifdef HAVE_SYS_SOCKET_H
 # ifndef SLRN_LOADED_SYS_SOCKET_H
 #  include <sys/socket.h>
@@ -138,45 +162,17 @@
 # endif
 #endif
 
-#ifdef VMS
- 
-#  ifdef MULTINET
-#   define netread   socket_read
-#   define sock_write  socket_write
-#   define netclose  socket_close
-#  else
-#   if NETLIB
-#    define netread   netlib_read
-#    define sock_write  netlib_write
-#    define netclose  netlib_close
-#   else
-#    define netread   read
-#    define sock_write  write
-#    define netclose  close
-#   endif
-#  endif
-# else
-# endif /* VMS */
-
-#ifndef NETLIB
-# define SERVER_READ_FP  ser_rd_fp
-# define SERVER_WRITE_FP ser_wr_fp
-#else
-# define SERVER_READ_FP	ipctx
-# define SERVER_WRITE_FP	ipctx
+#ifdef __unix__
+# ifdef MULTINET
+#  undef MULTINET
+# endif
+# ifdef NETLIB
+#  undef NETLIB
+# endif
 #endif
 
-#define HAS_BASIC_MEM_STUFF
-#ifdef HAS_BASIC_MEM_STUFF
-# ifndef bcopy
-#  define        bcopy(a,b,c)   memcpy(b,a,c)
-# endif
-# ifndef bzero
-#  define        bzero(a,b)     memset(a,'\0',b)
-# endif
-#endif /* USG */
-
 #ifdef EXCELAN
+/*{{{ EXCELAN defines */
 # define	NNTP_PORT	((unsigned short) 119)
 # if __STDC__
 int connect(int, struct sockaddr *);
@@ -185,6 +181,7 @@ unsigned long rhost(char **);
 int rresvport( int );
 int socket( int, struct sockproto *, struct sockaddr_in *, int );
 # endif
+/*}}}*/
 #endif
 
 #ifdef DECNET
@@ -192,6 +189,8 @@ int socket( int, struct sockproto *, struct sockaddr_in *, int );
 # include <netdnet/dnetdb.h>
 #endif /* DECNET */
 
+#include "clientlib.h"
+
 #include "server.h"
 #include "misc.h"
 
@@ -207,18 +206,12 @@ int socket( int, struct sockproto *, struct sockaddr_in *, int );
 #endif
 
 
-#ifdef VMS
-# ifdef NETLIB
-static void *ipctx;
-# else
-static int ser_rd_fp = -1, ser_wr_fp = -1;
-# endif
+#ifdef NETLIB
+static void *Socket_Fd;
 #else
-static FILE	*ser_rd_fp = NULL;
-static FILE	*ser_wr_fp = NULL;
+static int Socket_Fd;
 #endif
 
-
 #ifdef DEBUG_FILE
 static FILE *Debug_Fp;
 #endif
@@ -227,17 +220,11 @@ static int Server_Has_Been_Closed = 1;
 
 
 static void client_close_server(void);
-static int client_get_server(char *, int);
 
-#ifdef VMS
-static void cancel_read (int sig_unused)
-{
-   slrn_exit_error("\nCANCEL_READ: NNTP read failed.");
-}
-#endif
-
-static int get_tcp_socket(char *);
+static int get_tcp_socket(char *, int);
+static void reset_read_buffer (void);
 
+/*{{{ client_server_init_1 */
 /*
  * client_server_init  Get a connection to the remote news server.
  *
@@ -247,19 +234,14 @@ static int get_tcp_socket(char *);
  *			server's initial response code on success.
  *
  *	Side effects:	Connects to server.
- *			"ser_rd_fp" and "ser_wr_fp" are fp's
- *			for reading and writing to server.
  */
 
-static int client_server_init(char *machine, char *line, int len)
+static int client_server_init_1 (char *machine, int port)
 {
-   int	sockt_rd, sockt_wr;
+   int	sockt_rd;
 #ifdef DECNET
    char	*cp;
    
-   if (Server_Has_Been_Closed == 0) 
-     client_close_server ();
-   
    cp = slrn_strchr(machine, ':');
    
    if (cp && cp[1] == ':') 
@@ -268,63 +250,30 @@ static int client_server_init(char *machine, char *line, int len)
 	sockt_rd = get_dnet_socket(machine);
      }
    else
-     sockt_rd = get_tcp_socket(machine);
+     sockt_rd = get_tcp_socket(machine, port);
 #else
-   if (Server_Has_Been_Closed == 0) 
-     client_close_server ();
-   sockt_rd = get_tcp_socket(machine);
+   sockt_rd = get_tcp_socket (machine, port);
 #endif
    
    if (sockt_rd < 0)
      return (-1);
-#ifdef VMS
-# ifndef NETLIB
-   ser_rd_fp = ser_wr_fp = sockt_rd;
-# endif
-#else
-   /*
-    * Now we'll make file pointers (i.e., buffered I/O) out of
-    * the socket file descriptor.  Note that we can't just
-    * open a fp for reading and writing -- we have to open
-    * up two separate fp's, one for reading, one for writing.
-    */
-   
-   if ((ser_rd_fp = fdopen(sockt_rd, "r")) == NULL) 
-     {
-	perror("client_server_init: fdopen #1");
-	return (-1);
-     }
-   
-   sockt_wr = dup(sockt_rd);
-# ifdef TLI
-   if (t_sync(sockt_rd) < 0)
-     {
-	/* Sync up new fd with TLI */
-	t_error("client_server_init: t_sync");
-	ser_rd_fp = NULL;		/* from above */
-	return (-1);
-     }
-# endif
-   if ((ser_wr_fp = fdopen(sockt_wr, "w")) == NULL) 
-     {
-	perror("client_server_init: fdopen #2");
-	ser_rd_fp = NULL;		/* from above */
-	return (-1);
-     }
-#endif /* VMS */
+
+#ifndef NETLIB
+   Socket_Fd = sockt_rd;
+#endif
    
    /* Now get the server's signon message */
 #ifdef DEBUG_FILE
    Debug_Fp = fopen (DEBUG_FILE, "w");
 #endif
    
-   Server_Has_Been_Closed = 0;
-   (void) client_get_server(line, len);
-   
-   return (atoi(line));
+   return 0;
 }
 
 
+/*}}}*/
+
+/*{{{ get_tcp_socket */
 /*
  * get_tcp_socket -- get us a socket connected to the news server.
  *
@@ -338,8 +287,7 @@ static int client_server_init(char *machine, char *line, int len)
  *	Errors:		Printed via perror.
  */
 
-static int get_tcp_socket(machine)
-char	*machine;	/* remote host */
+static int get_tcp_socket(char *machine, int nntpport)
 {
 #ifndef NETLIB
    int	s = -1;
@@ -362,11 +310,11 @@ char	*machine;	/* remote host */
 	t_close(s);
 	return(-1);
      }
-   bzero((char *) &s_in, sizeof(s_in));
+   memset ((char *) &s_in, 0, sizeof(s_in));
    s_in.sin_family = AF_INET;
-   
-   if (Slrn_NNTP_Port < 0) Slrn_NNTP_Port = NNTP_PORT;
-   s_in.sin_port = htons((unsigned short) Slrn_NNTP_Port);
+
+   if (nntpport < 0) nntpport = NNTP_PORT;
+   s_in.sin_port = htons((unsigned short) nntpport);
    
    if (!isdigit(*machine) ||
        (long)(s_in.sin_addr.s_addr = inet_addr(machine)) == -1)
@@ -379,7 +327,7 @@ char	*machine;	/* remote host */
 	     t_close(s);
 	     return(-1);
 	  }
-	bcopy(hp->h_addr, (char *) &s_in.sin_addr, hp->h_length);
+	memcpy ((char *) &s_in.sin_addr, hp->h_addr, hp->h_length);
      }
    /*
     * Allocate a t_call structure and initialize it.
@@ -436,11 +384,11 @@ char	*machine;	/* remote host */
    struct SINDEF s_in;
    unsigned int rc, size;
    
-   if (Slrn_NNTP_Port == -1) Slrn_NNTP_Port = NNTP_PORT;
-   nntp_port = (unsigned short) Slrn_NNTP_Port;
+   if (nntpport < 0) nntpport = NNTP_PORT;
+   nntp_port = (unsigned short) nntpport;
    port = netlib_hton_word(&nntp_port);
    
-   rc = netlib_socket(&ipctx);
+   rc = netlib_socket(&Socket_Fd);
    if (!$VMS_STATUS_SUCCESS(rc))
      return(-1);
    if (isdigit(*machine))
@@ -454,20 +402,20 @@ char	*machine;	/* remote host */
 	s_in.sin_x_addr.inaddr_l_addr = addr.inaddr_l_addr;
 	memset(&s_in.sin_x_mbz, 0, 8);
 	size = sizeof(struct SINDEF);
-	rc = netlib_connect(&ipctx, &s_in, &size, 0,0,0);
+	rc = netlib_connect(&Socket_Fd, &s_in, &size, 0,0,0);
 	if (!$VMS_STATUS_SUCCESS(rc))
 	  {
-	     netlib_close(&ipctx);
+	     netlib_close(&Socket_Fd);
 	     return(-1);
 	  }
      }
    else
      {
 	INIT_SDESC(ndsc, strlen(machine), machine);
-	rc = netlib_connect_by_name(&ipctx, &ndsc, &nntp_port);
+	rc = netlib_connect_by_name(&Socket_Fd, &ndsc, &nntp_port);
 	if (!$VMS_STATUS_SUCCESS(rc))
 	  {
-	     netlib_close(&ipctx);
+	     netlib_close(&Socket_Fd);
 	     return(-1);
 	  }
      }
@@ -493,11 +441,11 @@ char	*machine;	/* remote host */
 #   ifndef UCX
    struct servent sp_buffer;
    
-   if (Slrn_NNTP_Port >= 0)
+   if (nntpport >= 0)
      {
 	sp = &sp_buffer;
 	memset ((char *) sp, 0, sizeof (struct servent));
-	sp->s_port = htons((unsigned short)Slrn_NNTP_Port);
+	sp->s_port = htons((unsigned short)nntpport);
      }
    else 
      {
@@ -508,9 +456,9 @@ char	*machine;	/* remote host */
 	  }
      }
 #   else
-   if (Slrn_NNTP_Port == -1) Slrn_NNTP_Port = NNTP_PORT;
+   if (nntpport < 0) nntpport = NNTP_PORT;
    sp = (struct servent *)malloc(sizeof(struct servent));
-   sp->s_port = htons((unsigned short)Slrn_NNTP_Port);
+   sp->s_port = htons((unsigned short)nntpport);
 #   endif
    /* If not a raw ip address, try nameserver */
    if (!isdigit(*machine) ||
@@ -536,11 +484,11 @@ char	*machine;	/* remote host */
 	return (-1);
      }
    
-   bzero((char *) &s_in, sizeof(s_in));
+   memset ((char *) &s_in, 0, sizeof(s_in));
    s_in.sin_family = hp->h_addrtype;
    s_in.sin_port = sp->s_port;
 #  else /* EXCELAN */
-   bzero((char *) &s_in, sizeof(s_in));
+   memset ((char *) &s_in, 0, sizeof(s_in));
    s_in.sin_family = AF_INET;
 #  endif /* EXCELAN */
    
@@ -566,7 +514,7 @@ char	*machine;	/* remote host */
 	     perror("socket");
 	     return (-1);
 	  }
-	bcopy(*cp, (char *)&s_in.sin_addr, hp->h_length);
+	memcpy ((char *)&s_in.sin_addr, *cp, hp->h_length);
 	
 	if (x < 0)
 	  fprintf(stderr, "trying %s\n",
@@ -592,10 +540,10 @@ char	*machine;	/* remote host */
 	perror("socket");
 	return (-1);
      }
-   bzero((char *) &s_in, sizeof(s_in));
+   memset ((char *) &s_in, 0, sizeof(s_in));
    s_in.sin_family = AF_INET;
-   if (Slrn_NNTP_Port == -1) Slrn_NNTP_Port = NNTP_PORT;
-   s_in.sin_port = htons((unsigned short) Slrn_NNTP_Port);
+   if (nntpport < 0) nntpport = NNTP_PORT;
+   s_in.sin_port = htons((unsigned short) nntpport);
    /* set up addr for the connect */
    
    if ((s_in.sin_addr.s_addr = rhost(&machine)) == -1) 
@@ -620,7 +568,7 @@ char	*machine;	/* remote host */
    
    /* And then connect */
    
-   bcopy(hp->h_addr, (char *) &s_in.sin_addr, hp->h_length);
+   memcpy ((char *) &s_in.sin_addr, hp->h_addr, hp->h_length);
    if (connect(s, (struct sockaddr *) &s_in, sizeof(s_in)) < 0) 
      {
 	perror("connect");
@@ -638,7 +586,10 @@ char	*machine;	/* remote host */
 #endif
 }
 
+/*}}}*/
+
 #ifdef DECNET
+/*{{{ get_dnet_socket */
 /*
  * get_dnet_socket -- get us a socket connected to the news server.
  *
@@ -652,14 +603,13 @@ char	*machine;	/* remote host */
  *	Errors:		Printed via nerror.
  */
 
-get_dnet_socket(machine)
-char	*machine;
+int get_dnet_socket (char *machine)
 {
    int	s, area, node;
    struct	sockaddr_dn sdn;
    struct	nodeent *getnodebyname(), *np;
    
-   bzero((char *) &sdn, sizeof(sdn));
+   memset ((char *) &sdn, 0, sizeof(sdn));
    
    switch (s = sscanf( machine, "%d%*[.]%d", &area, &node )) 
      {
@@ -682,8 +632,7 @@ char	*machine;
 	  }
 	else 
 	  {
-	     bcopy(np->n_addr,
-		   (char *) sdn.sdn_add.a_addr,
+	     memcpy((char *) sdn.sdn_add.a_addr, np->n_addr,
 		   np->n_length);
 	     sdn.sdn_add.a_len = np->n_length;
 	     sdn.sdn_family = np->n_addrtype;
@@ -693,7 +642,7 @@ char	*machine;
    sdn.sdn_objnum = 0;
    sdn.sdn_flags = 0;
    sdn.sdn_objnamel = strlen("NNTP");
-   bcopy("NNTP", &sdn.sdn_objname[0], sdn.sdn_objnamel);
+   memcpy (&sdn.sdn_objname[0], "NNTP", sdn.sdn_objnamel);
    
    if ((s = socket(AF_DECnet, SOCK_STREAM, 0)) < 0) 
      {
@@ -712,328 +661,334 @@ char	*machine;
    
    return (s);
 }
+/*}}}*/
 #endif
 
+/*{{{ Low-Level Read Routines */
 
+#define TIMEOUT 60
+#define BUFSIZE 8192
 
-/*
- * handle_server_response
- *
- *	Print some informative messages based on the server's initial
- *	response code.  This is here so inews, rn, etc. can share
- *	the code.
- *
- *	Parameters:	"response" is the response code which the
- *			server sent us, presumably from "client_server_init",
- *			above.
- *			"nntpserver" is the news server we got the
- *			response code from.
- *
- *	Returns:	-1 if the error is fatal (and we should exit).
- *			0 otherwise.
- *
- *	Side effects:	None.
- */
-#if 0
-static int handle_server_response(int response, char *nntpserver)
+static unsigned char Read_Buffer [BUFSIZE];
+static unsigned char *Read_Buffer_Ptr = Read_Buffer;
+static unsigned char *Read_Buffer_Ptr_Max = Read_Buffer;
+
+static void reset_read_buffer (void)
 {
-   switch (response) 
-     {
-      case OK_NOPOST:		/* fall through */
-	printf(
-	       "NOTE: This machine does not have permission to post articles.\n");
-	printf(
-	       "      Please don't waste your time trying.\n\n");
-	
-      case OK_CANPOST:
-	return (0);
-	/* break; */
-	
-      case ERR_ACCESS:
-	printf(
-	       "This machine does not have permission to use the %s news server.\n",
-	       nntpserver);
-	return (-1);
-	/* break; */
-	
-      default:
-	printf("Unexpected response code from %s news server: %d\n",
-	       nntpserver, response);
-	return (-1);
-	/* break; */
-     }
-   /*NOTREACHED*/
+   Read_Buffer_Ptr_Max = Read_Buffer_Ptr = Read_Buffer;
 }
 
-#endif
-/*
- * client_put_server -- send a line of text to the server, terminating it
- * with CR and LF, as per ARPA standard.
- *
- *	Parameters:	"string" is the string to be sent to the
- *			server.
- *
- *	Returns:	Nothing.
- *
- *	Side effects:	Talks to the server.
- *
- *	Note:		This routine flushes the buffer each time
- *			it is called.  For large transmissions
- *			(i.e., posting news) don't use it.  Instead,
- *			do the fprintf's yourself, and then a final
- *			fflush.
- */
-
-static int
-client_put_server(char *string)
+static void cancel_read (int sig_unused)
 {
-#ifdef VMS
-   char *cp;
-# ifdef NETLIB
-   int rc;
-   struct dsc$descriptor dsc;
-   unsigned short retlen;
-# endif
+   (void) sig_unused;
+   slrn_error ("CANCEL_READ: NNTP read failed.");
+}
+
+
+static int client_read (void)
+{
+   int nb;
+#if TIMEOUT
+   void (*old_sigalrm)(int);
 #endif
    
-   if (Server_Has_Been_Closed)
-     return -1;
-   
-#ifdef DEBUG_FILE
-   if (Debug_Fp != NULL) fprintf(Debug_Fp, ">>> %s\n", string);
+   reset_read_buffer ();
+#if TIMEOUT
+   old_sigalrm = signal (SIGALRM, cancel_read);
+   alarm (TIMEOUT);
 #endif
    
-#ifdef VMS
-# ifndef NETLIB
-   cp = (char *)malloc(strlen(string)+4);
-   sprintf(cp, "%s\r\n", string);
-   netwrite(SERVER_WRITE_FP, cp, strlen(cp));
-   /* sock_write(ser_wr_fp, cp, strlen(cp)); */
-   free(cp);
-# else
-   INIT_SDESC(dsc, strlen(string), string);
-   rc = netlib_writeline(&ipctx, &dsc);
-   if (!(rc & 1)) return(-1);
-# endif /* !NETLIB */
+#ifdef NETLIB
+   nb = do_netlib_read (Socket_Fd, Read_Buffer, BUFSIZE);
 #else
-   if ((EOF == fputs (string, ser_wr_fp))
-       || (EOF == fputs ("\r\n", ser_wr_fp))
-       || (EOF == fflush(ser_wr_fp)))
-     return -1;
+# ifdef MULTINET
+   nb = socket_read (Socket_Fd, Read_Buffer, BUFSIZE);
+# else
+   while (-1 == (nb = read (Socket_Fd, Read_Buffer, BUFSIZE)))
+     {
+#  ifdef EINTR
+	if (errno == EINTR) continue;
+#  endif
+	break;
+     }   
+# endif				       /* MULTINET */
+#endif				       /* NETLIB */
+   
+#if TIMEOUT
+   alarm(0);
+   (void) signal (SIGALRM, old_sigalrm);
 #endif
+   
+   if (nb <= 0)
+     return -1;
+   
+   Read_Buffer_Ptr_Max = Read_Buffer + nb;
+   
    return 0;
 }
 
 
-/*
- * client_get_server -- get a line of text from the server.  Strips
- * CR's and LF's.
- *
- *	Parameters:	"string" has the buffer space for the
- *			line received.
- *			"size" is the size of the buffer.
- *
- *	Returns:	-1 on error, 0 otherwise.
- *
- *	Side effects:	Talks to server, changes contents of "string".
- */
-
-#if defined(VMS) && !defined(NETLIB)
-# define TIMEOUT 60
-# define BUFSIZE 8192
-static char buffer[BUFSIZE];
-static int  bufread = 0;
-
-static int client_get_server(char *string, int size)
+static int client_fgets (char *buf, int len)
 {
-   register char *cp;
-   int status, nb, len, count;
-   struct 
-     {
-	unsigned short status;
-	unsigned short count;
-	unsigned int undef;
-     }
-   r_iosb;
+   unsigned char *r, *rmax;
+   char *buf_max;
    
    if (Server_Has_Been_Closed) return -1;
    
-   if (!bufread)
+   if (len == 0)
      {
-	signal(SIGALRM,cancel_read);
-	alarm(TIMEOUT);
-	nb = netread(SERVER_READ_FP, buffer, BUFSIZE);
-	alarm(0);
-	if (nb <= 0)
-	  return(-1);
-	bufread = nb;
-	buffer[bufread] = '\0';
-# ifdef DEBUG_FILE
-	if (Debug_Fp != NULL) fprintf(Debug_Fp, "<<<<< %s\n", buffer);
-# endif
+	*buf = 0;
+	return 0;
      }
-   *string = '\0';
-   count=0;
-   while (((cp = strstr(buffer, "\r\n")) == NULL) && (count < 5))
+   
+   buf_max = buf + (len - 1);	       /* allow room for \0 */
+   
+   while (1)
      {
-	count++;
-	strncpy(string, buffer, strlen(buffer));
-	string[strlen(buffer)] = '\0';
-	bufread=0;
-	memset(buffer, 0, BUFSIZE);
-	signal(SIGALRM,cancel_read);
-	alarm(TIMEOUT);
-	nb = netread(SERVER_READ_FP, buffer, BUFSIZE);
-	alarm(0);
-	if (nb <= 0)
+	r = Read_Buffer_Ptr;
+	rmax = Read_Buffer_Ptr_Max;
+   
+	while (r < rmax)
 	  {
-# ifdef DEBUG_FILE
-	     if (Debug_Fp != NULL) fprintf(Debug_Fp, "**** %s\n", string);
-# endif
-	     return(0);
+	     unsigned char ch;
+	     
+	     if (buf == buf_max)
+	       {
+		  *buf = 0;
+		  Read_Buffer_Ptr = r;
+		  return 0;
+	       }
+	     
+	     ch = *r++;
+	     *buf++ = (char) ch;
+
+	     if (ch == '\n')
+	       {
+		  *buf = 0;
+		  Read_Buffer_Ptr = r;
+		  return 0;
+	       }
+
 	  }
-	bufread = nb;
-	buffer[bufread] = '\0';
+	     
+	if (-1 == client_read ())
+	  return -1;
      }
-   /* while */
-   *cp = '\0'; cp += 2;
-   bufread = strlen(cp);
-   strcat(string, buffer);
-   if (bufread > 0)
-     {
-	memmove(buffer, cp, bufread);
-	buffer[bufread] = '\0';
-     }
-   else
-     memset(buffer, 0, BUFSIZE);
-# ifdef DEBUG_FILE
-   if (Debug_Fp != NULL) fprintf(Debug_Fp, "<<< %s\n", string);
-# endif
-   return (0);
 }
 
+/*}}}*/
 
-#else /* !VMS || NETLIB */
-# ifdef NETLIB
-#  define TIMEOUT 60
-static int client_get_server(char *string, int size)
+
+static void client_close_server(void) /*{{{*/
 {
-   register char *cp;
-   int rc;
-   struct dsc$descriptor dsc;
-   unsigned short retlen;
-   
-   INIT_SDESC(dsc, size, string);
-   if (Server_Has_Been_Closed) return -1;
-   rc = netlib_readline(&ipctx, &dsc, &retlen, 0, 0);
-   if (!(rc & 1))
-     return (-1);
-   
-   string[retlen] = '\0';
-#  ifdef DEBUG_FILE
-   if (Debug_Fp != NULL) fprintf(Debug_Fp, "<<< %s\n", string);
-#  endif
-   
-   return (0);
-}
-
-# else /* !VMS && !NETLIB */
-static int client_get_server(char *string, int size)
-{
-   register char *cp;
-   
-   if (Server_Has_Been_Closed) return -1;
-
-   if (fgets(string, size, ser_rd_fp) == NULL)
-     return (-1);
-   
-   if ((cp = slrn_strchr(string, '\r')) != NULL)
-     *cp = '\0';
-   else if ((cp = slrn_strchr(string, '\n')) != NULL)
-     *cp = '\0';
-   
-#  ifdef DEBUG_FILE
-   if (Debug_Fp != NULL) fprintf(Debug_Fp, "<<< %s\n", string);
-#  endif
-   
-   return (0);
-}
-# endif /* !NETLIB */
-#endif /* !VMS || NETLIB */
-
-/*
- * client_close_server -- close the connection to the server, after sending
- *		the "quit" command.
- *
- *	Parameters:	None.
- *
- *	Returns:	Nothing.
- *
- *	Side effects:	Closes the connection with the server.
- *			You can't use "client_put_server" or "client_get_server"
- *			after this routine is called.
- */
-
-static void
-client_close_server(void)
-{
-   char	ser_line[256];
+#if defined(VMS) && !defined(NETLIB)
+   void (*save_sigalrm)(int);
+#endif
    
    if (Server_Has_Been_Closed) return;
 
-#ifdef VMS
-# ifndef NETLIB
-   if (ser_wr_fp <= 0 || ser_rd_fp <= 0) return;
-# else
-   if (ipctx == NULL) return;
-# endif
+   if (
+#ifdef NETLIB
+       (Socket_Fd == NULL)
 #else
-   if (ser_wr_fp == NULL || ser_rd_fp == NULL)
-     return;
+       (Socket_Fd < 0)
 #endif
-   
-   if (-1 != client_put_server("QUIT"))
-     (void) client_get_server(ser_line, sizeof(ser_line));
+       )
+     {
+	Server_Has_Been_Closed = 1;
+	return;
+     }
    
 #ifdef NETLIB
-   netclose(&ipctx);
+   netlib_close (&Socket_Fd);
+   Socket_Fd = NULL;
 #else
-# ifdef VMS
-   netclose(ser_rd_fp);
-   netclose(ser_wr_fp);
+# ifdef MULTINET
+   save_sigalrm = signal (SIGALRM, cancel_read);
+   alarm (10);
+   socket_close (Socket_Fd);
+   alarm (0);
+   signal (SIGALRM, save_sigalrm);
 # else
-   (void) fclose(ser_wr_fp);
-   (void) fclose(ser_rd_fp);
-# endif
-#endif
-   Server_Has_Been_Closed = 1;
+   close (Socket_Fd);
+# endif				       /* MULTINET */
+   Socket_Fd = -1;
+#endif				       /* NETLIB */
 }
 
-#ifdef VMS
-# ifdef NETLIB
-static int netwrite(void *sockt, char *string, int len)
+/*}}}*/
+
+static unsigned int client_write (char *buf, unsigned int len)
 {
-   struct dsc$descriptor dsc;
-   int rc;
-   struct NETLIBIOSBDEF iosb;
+   int n;
+   unsigned int total;
    
-   INIT_SDESC(dsc, len, string);
-   netlib_write(&sockt, &dsc, 0,0, &iosb,0,0);
+#ifdef NETLIB
+   struct dsc$descriptor dsc;
+   struct NETLIBIOSBDEF iosb;
+   int rc;
+   
+   INIT_SDESC(dsc, len, buf);
+   netlib_write(&Socket_Fd, &dsc, 0,0, &iosb,0,0);
    if ($VMS_STATUS_SUCCESS(rc))
-     {
-	return(iosb.iosb_w_count);
-     }
-   else
-     return(-1);
-}
+     total = iosb.iosb_w_count;
+   else total = 0;
+#else
+# ifdef MULTINET
+   total = socket_write (Socket_Fd, buf, len);
 # else
-
-static int netwrite(int sockt, char *string, int len)
-{
-#  ifdef DEBUG_FILE
-   if (Debug_Fp != NULL) fprintf(Debug_Fp, ">>> %s\n", string);
+   total = 0;
+   while (len != 0)
+     {
+	n = write (Socket_Fd, buf + total, len);
+	if (n == -1)
+	  {
+#  ifdef EAGAIN
+	     if (errno == EAGAIN) 
+	       {
+		  sleep (1);
+		  continue;
+	       }
 #  endif
-   return(sock_write(sockt, string, len));
+#  ifdef EWOULDBLOCK
+	     if (errno == EWOULDBLOCK)
+	       {
+		  sleep (1);
+		  continue;
+	       }
+#  endif
+#  ifdef EINTR
+	     if (errno == EINTR)
+	       continue;
+#  endif
+	     break;
+	  }
+	len -= n;
+	total += n;
+     }
+# endif				       /* MULTINET */
+#endif				       /* NETLIB */
+   
+#ifdef DEBUG_FILE
+   if ((Debug_Fp != NULL) && (total != 0))
+     {
+	fputs (buf, Debug_Fp);
+	fflush (Debug_Fp);
+     }
+#endif
+   
+   return total;
 }
-# endif  /* !NETLIB */
-#endif  /* VMS */
+
+
+static int client_fputs (char *s)
+{
+   unsigned int len = strlen (s);
+   
+   if (len != client_write (s, len))
+     return -1;
+   
+   return 0;
+}
+
+static int client_vprintf (char *fmt, va_list ap)
+{
+   char buf [8192];
+   vsprintf (buf, fmt, ap);
+   return client_fputs (buf);
+}
+
+static int client_server_init (char *machine, int port)
+{
+   if (Server_Has_Been_Closed == 0) 
+     client_close_server ();
+
+   reset_read_buffer ();
+   
+   if (-1 == client_server_init_1 (machine, port))
+     return -1;
+   
+   Server_Has_Been_Closed = 0;
+   return 0;
+}
+
+
+static int client_flush_output (void)
+{
+   return 0;
+}
+
+
+static SLTCP_Type SLTCP_Buffer;
+
+int sltcp_fputs (SLTCP_Type *s, char *buf)
+{
+   if (s == NULL) return -1;
+   return client_fputs (buf);
+}
+
+int sltcp_vfprintf (SLTCP_Type *s, char *fmt, va_list ap)
+{
+   if (s == NULL) return -1;
+   return client_vprintf (fmt, ap);
+}
+
+extern int sltcp_fgets (SLTCP_Type *s, char *buf, unsigned int len)
+{
+   if (s == NULL) return -1;
+   return client_fgets (buf, (int) len);
+}
+
+extern int sltcp_flush_output (SLTCP_Type *s)
+{
+   if (s == NULL) return -1;
+   return client_flush_output ();
+}
+
+extern int sltcp_close (SLTCP_Type *s)
+{
+   if (s == NULL) return -1;
+   
+   (void) client_flush_output ();
+   (void) client_close_server ();
+
+   SLTCP_Buffer.tcp_fd = -1;
+   
+   return 0;
+}
+
+
+extern unsigned int sltcp_write (SLTCP_Type *s, char *buf, unsigned int len)
+{
+   if (s == NULL) return 0;
+   
+   return client_write (buf, len);
+}
+
+
+extern SLTCP_Type *sltcp_open_connection (char *host, int port)
+{
+   if (-1 == client_server_init (host, port))
+     return NULL;
+   
+   SLTCP_Buffer.tcp_fd = 1;
+   return &SLTCP_Buffer;
+}
+
+extern int sltcp_map_service_to_port (char *srv)
+{
+   (void) srv;
+   return NNTP_PORT;
+}
+
+extern int sltcp_close_socket (SLTCP_Type *s)
+{
+   if (s == NULL) return -1;
+   client_close_server ();
+   
+   SLTCP_Buffer.tcp_fd = -1;
+
+   return 0;
+}
+
+int (*SLTCP_Interrupt_Hook) (void);
