@@ -2,15 +2,21 @@
  *  All rights reserved.
  */
 #include "config.h"
-#include "features.h"
+#include "slrnfeat.h"
 
 #include <stdio.h>
 #include <string.h>
 
+#ifdef HAVE_STDLIB_H
+# include <stdlib.h>
+#endif
 
 #include <slang.h>
+#include "jdmacros.h"
+
 #include "slrn.h"
 #include "misc.h"
+#include "util.h"
 
 static char *Global_Help [] = 
 {
@@ -32,6 +38,7 @@ static char *Art_Help[] =
      "  S           Subject search backward.",
      "  =           Skip to next article of specified subject",
      "  !           Skip to next article with high score",
+     "  ESC-l       Locate header by message-id.",
      "Commands that affect the article:",
      "  h           Hide window (zoom header window)",
      "  SPACE       Select article or scroll to next page",
@@ -47,6 +54,9 @@ static char *Art_Help[] =
      "  ESC-r       Toggle ROT 13 decryption on and off.",
      "  g           Skip to next digest article.",
      "  ESC Ctrl-C  Cancel article (you must be the author).",
+#if SLRN_HAS_GROUPLENS
+     "  0           Assign article a GroupLens rating.",
+#endif
      "Posting, Replying, Piping, and Saving:",
      "  P           Post a new article (NOT followup)",
      "  f           Followup to current article.",
@@ -81,7 +91,7 @@ static char *Art_Help[] =
      "  Ctrl-U      Perform Page-Up on the header window.",
      "  Ctrl-D      Perform Page-Dn on the header window.",
      "  ESC-a       Toggle header display format",
-     "  ESC-s       Toggle header threading on/off.",
+     "  ESC-s       Select sort mode",
      "  ESC-p       Find parent header reading from server if necessary",
      "  ESC 1 ESC p Reconstruct thread.",
      "  ESC Ctrl-P  Find all children of current header (queries server)",
@@ -90,9 +100,10 @@ static char *Art_Help[] =
      "  C           Mark articles TO the current position as read",
      "  ESC-U       Mark articles TO current position as UN-read",
      "  K           Create a score entry based on current header.",
-     "  ESC 1 K     Edit score file"
+     "  ESC 1 K     Edit score file",
      "  *           Mark header so it is un-affected by catchup commands.",
      "  ESC 1 *     Un-mark all headers marked with '*'.",
+     "  x           Remove all read articles from list.",
      "Miscellaneous:",
      "  . (Period)  Repeat last key sequence",
      "  U           Search for URL and follow it",
@@ -136,7 +147,7 @@ static char *Group_Help [] =
      "  Ctrl-Z               Suspend newsreader.",
      "  l                    Toggle listing of groups that have no unread articles.",
      "  L                    Toggle listing of unsubscribed groups.",
-     "  ESC 1 L              Hide unsubscribed groups"
+     "  ESC 1 L              Hide unsubscribed groups",
      "  /                    Group keyword search.  Use 'l' to search hidden groups.",
      "  q                    Quit newsreader",
      "  G                    Get new news",
@@ -144,6 +155,7 @@ static char *Group_Help [] =
      "  X                    Force a save of the newsrc file.",
      "  ESC-a                Toggle display of group description on/off.",
      "  Ctrl-X Ctrl-T        Transpose position of groups.",
+     "  m                    Move newsgroup to different location.",
      "  . (Period)           Repeat last key sequence",
      NULL
 };
@@ -161,12 +173,17 @@ static void do_help (char **help)
    
    this_help = p = Global_Help;
    
+   slrn_enable_mouse (0);
+   
    while (1)
      {
-	SLsmg_cls ();
 	i = 0;
 	if (*p == NULL) break;
 	
+	slrn_push_suspension (0);
+
+	SLsmg_cls ();
+	
 	if ((sect != NULL) && (**p == ' '))
 	  {
 	     SLsmg_set_color (1);
@@ -228,6 +245,9 @@ Press 'q' to quit help, '?' to start over, or any other key to continue."
 				 );
 	
 	slrn_smg_refresh ();
+	
+	slrn_pop_suspension ();
+
 	SLang_flush_input ();
 	quit = SLang_getkey ();
 	if (quit == '?')
@@ -239,8 +259,10 @@ Press 'q' to quit help, '?' to start over, or any other key to continue."
 	    break;
      }
    Slrn_Full_Screen_Update = 1;
-   slrn_redraw ();
+   slrn_set_color (0);
+   /* slrn_redraw (); */
    SLang_flush_input ();
+   slrn_enable_mouse (1);
 }
 
 int slrn_parse_helpfile (char *helpfile)
@@ -251,6 +273,9 @@ int slrn_parse_helpfile (char *helpfile)
    char **current_help = NULL;
    int num_lines = 0;
    unsigned char *b;
+
+   if (Slrn_Batch)
+     return 0;
    
    if (NULL == (fp = fopen (helpfile, "r"))) return -1;
    while (fgets (buf, sizeof (buf) - 1, fp) != NULL)
@@ -269,8 +294,7 @@ int slrn_parse_helpfile (char *helpfile)
 	     /* end current help */
 	     if (current_help != NULL)
 	       {
-		  if (current_help[num_lines] != NULL)
-		    SLFREE (current_help[num_lines]);
+		  slrn_free (current_help[num_lines]);
 		  current_help[num_lines] = NULL;
 	       }
 	     
@@ -295,19 +319,14 @@ int slrn_parse_helpfile (char *helpfile)
 	     continue;
 	  }
 	
-	if (current_help [num_lines] != NULL)
-	  SLFREE (current_help[num_lines]);
+	slrn_free (current_help [num_lines]);
 	
-	if (NULL != (current_help [num_lines] = (char *) SLMALLOC (strlen (buf) + 1)))
-	  {
-	     strcpy (current_help[num_lines], buf);
-	     num_lines++;
-	  }
+	if (NULL != (current_help [num_lines] = (char *) slrn_strmalloc (buf, 0)))
+	  num_lines++;
      }
    if (current_help != NULL)
      {
-	if (current_help[num_lines] != NULL)
-	  SLFREE (current_help[num_lines]);
+	slrn_free (current_help[num_lines]);
 	current_help[num_lines] = NULL;
      }
    slrn_fclose (fp);
@@ -318,6 +337,7 @@ int slrn_parse_helpfile (char *helpfile)
 void slrn_article_help (void)
 {
    char **h;
+   if (Slrn_Batch) return;
    if (User_Article_Help[0] != NULL) h = User_Article_Help; else h = Art_Help;
    do_help (h);
 }
@@ -325,6 +345,7 @@ void slrn_article_help (void)
 void slrn_group_help (void)
 {
    char **h;
+   if (Slrn_Batch) return;
    if (User_Group_Help[0] != NULL) h = User_Group_Help; else h = Group_Help;
    do_help (h);
 }
