@@ -5,7 +5,7 @@
  */
 
 #include "config.h"
-#include "features.h"
+#include "slrnfeat.h"
 
 #include <stdio.h>
 #include <string.h>
@@ -21,13 +21,20 @@
 
 #include <ctype.h>
 
+#if defined (__os2__)
+# include <process.h>
+#endif
+
 #include <slang.h>
+#include "jdmacros.h"
+
 #include "server.h"
 #include "slrn.h"
 #include "misc.h"
 #include "slrn.h"
 #include "group.h"
 #include "art.h"
+#include "util.h"
 
 #if SLRN_HAS_MIME
 /* rest of file in this ifdef */
@@ -36,6 +43,7 @@
 
 int Slrn_Use_Mime = 1;
 int Slrn_Use_Meta_Mail = 1;
+char *Slrn_MetaMail_Cmd;
 
 int Slrn_Mime_Was_Parsed;
 int Slrn_Mime_Was_Modified;
@@ -187,10 +195,10 @@ static int parse_content_type_line (void)
 	     
 	     if (0 != slrn_case_strncmp ((unsigned char *)b,
 					 (unsigned char *)"charset",
-					 8))
+					 7))
 	       continue;
 	     
-	     b = slrn_skip_whitespace (b + 8);
+	     b = slrn_skip_whitespace (b + 7);
 	     while (*b == 0)
 	       {
 		  line = line->next;
@@ -202,7 +210,7 @@ static int parse_content_type_line (void)
 	       }
 	     
 	     if (*b != '=') continue;
-	     
+	     b++;
 	     if (*b == '"') b++;
 	     charset = b;
 	     while (*b && (*b != ';')
@@ -271,11 +279,14 @@ static char *decode_quoted_printable (char *dest,
 				      char *src, char *srcmax,
 				      int treat_underscore_as_space)
 {
+   char *allowed_in_qp = "0123456789ABCDEFabcdef";
    char ch;
    while (src < srcmax)
      {
 	ch = *src++;
-	if ((ch == '=') && (src + 1 < srcmax))
+	if ((ch == '=') && (src + 1 < srcmax)
+	    && (NULL != slrn_strchr (allowed_in_qp, src[0]))
+	    && (NULL != slrn_strchr (allowed_in_qp, src[1])))
 	  {
 	     *dest++ = (16 * HEX(src[0])) + HEX(src[1]);
 	     src += 2;
@@ -553,19 +564,26 @@ int slrn_mime_call_metamail (void)
    return 0;
 #else
    int init = Slrn_TT_Initialized;
-   char cmd[MAXPATHLEN];
-   char tempfile[MAXPATHLEN];
+   char cmd [2 * SLRN_MAX_PATH_LEN];
+   char tempfile [SLRN_MAX_PATH_LEN];
    Slrn_Article_Line_Type *ptr = Slrn_Article_Lines;
    FILE *fp;
-   
+   char *tmp, *mm;
    
    if ((Slrn_Use_Meta_Mail == 0)
-       || slrn_get_yesno (1, "Process this MIME article with metamail") <= 0)
+       || Slrn_Batch
+       || (slrn_get_yesno (1, "Process this MIME article with metamail") <= 0))
      return 0;
+
+# ifdef __os2__
+   if (NULL == (tmp = getenv ("TMP")))
+     tmp = ".";
+#else
+   tmp = "/tmp";
+# endif
    
-   sprintf(tempfile, "/tmp/slrn.%ld", (long) getpid());
-   
-   fp = fopen(tempfile, "w");
+   fp = slrn_open_tmpfile_in_dir (tmp, tempfile, "w");
+
    if (fp == NULL)
      {
 	slrn_error ("Unable to open tmp file for metamail.");
@@ -579,21 +597,27 @@ int slrn_mime_call_metamail (void)
 	ptr = ptr->next;
      }
    slrn_fclose(fp);
+
+   mm = Slrn_MetaMail_Cmd;
    
-   strcpy(cmd, "metamail ");
-   strcat(cmd, tempfile);
+   if ((mm == NULL)
+       || (*mm == 0)
+       || (strlen (mm) > SLRN_MAX_PATH_LEN))
+     mm = "metamail";
+
+   sprintf (cmd, "%s %s", mm, tempfile);
    
    /* Make sure that metamail has a normal environment */
-   slrn_reset_display(0);
+   slrn_set_display_state (0);
    
    slrn_posix_system(cmd, 0);
    slrn_delete_file (tempfile);
    
-   printf("Press return to continue...");
+   printf("Press return to continue ...");
    getchar();
    fflush(stdin); /* get rid of any pending input! */
    
-   slrn_init_display(init, 0);
+   slrn_set_display_state (init);
    return 1;
 #endif  /* NOT VMS */
 }
@@ -690,7 +714,7 @@ int slrn_mime_scan_file (FILE *fp)
  * separated words and not arbitrary sequences of text.
  */
 static int rfc1522_encode (unsigned char *d, unsigned int len,
-			   unsigned char *s)
+			   unsigned char *s, int ignore_specials)
 {
    /* This routine encodes the a string containing 8-bit characters according
     * to the conventions of RFC1522.  The strategy here is to print up to
@@ -769,7 +793,8 @@ static int rfc1522_encode (unsigned char *d, unsigned int len,
 	  {
 	     p--;
 	     ch = *p;
-	     if (0 == isalnum(ch))
+	     if ((ch == ' ') || (ch == '\t')
+		 || ((ignore_specials == 0) && IS_RFC850_SPECIAL(ch)))
 	       {
 		  p++;
 		  break;
@@ -788,13 +813,20 @@ static int rfc1522_encode (unsigned char *d, unsigned int len,
 	
 	strcpy ((char *)d, charset);
 	d += charset_len;
-	
+
+	/* Note: rfc1522 places a limit on the length of the encoded word.  
+	 * That limit is ignored here.  Perhaps someday I will recode this to 
+	 * take the limit into account.
+	 */
 	while ((d < dmax) && ((ch = *p) != 0)
-	       && (0 == IS_RFC850_SPECIAL(ch))
-	       && ((hibit != 0) || isalnum(ch)))
+	       && ((ignore_specials != 0) || (0 == IS_RFC850_SPECIAL(ch)))
+	       && ((hibit != 0) 
+		   || ((ch != ' ') && (ch != '\t'))))
 	  {
 	     if (ch == ' ') *d++ = '_';
-	     else if ((ch & 0x80) || ((ch < 32) && (ch != '\n')))
+	     else if ((ch & 0x80) 
+		      || ((ch < 32) && (ch != '\n'))
+		      || IS_RFC850_SPECIAL(ch))
 	       {
 		  /* We need 3 characters to encode this. */
 		  if (d + 3 >= dmax)
@@ -822,10 +854,20 @@ void slrn_mime_header_encode (char *s, unsigned int bytes)
 {
    char buf[1024];
    unsigned int len = strlen (s);
+   
    if (len < sizeof (buf))
      {
+	int ignore_specials;
+	
+	/* Perhaps this test should be extended to include other headers. */
+	ignore_specials = !slrn_case_strncmp ((unsigned char *)s, 
+					      (unsigned char *)"Subject: ", 9);
+	
 	strcpy (buf, s);
-	if (0 == rfc1522_encode ((unsigned char *)s, bytes, (unsigned char *)buf)) return;
+	if (0 == rfc1522_encode ((unsigned char *)s, bytes, 
+				 (unsigned char *)buf,
+				 ignore_specials))
+	    return;
      }
    
    /* Cannot do it so strip it to 8 bits. */
@@ -871,10 +913,10 @@ Content-Transfer-Encoding: %s\n",
      }
    else
      {
-	Slrn_Post_Obj->printf ("\
-Mime-Version: 1.0\r\n\
-Content-Type: text/plain; charset=%s\r\n\
-Content-Transfer-Encoding: %s\r\n",
+	Slrn_Post_Obj->po_printf ("\
+Mime-Version: 1.0\n\
+Content-Type: text/plain; charset=%s\n\
+Content-Transfer-Encoding: %s\n",
 		 Mime_Posting_Charset,
 		 encoding);
      }
