@@ -3,7 +3,7 @@
  */
 /* post an article */
 #include "config.h"
-#include "features.h"
+#include "slrnfeat.h"
 
 #include <stdio.h>
 #include <string.h>
@@ -20,14 +20,22 @@
 #include <sys/types.h>
 #include <time.h>
 #include <slang.h>
+#include "jdmacros.h"
+
+
+#ifdef VMS
+# include "vms.h"
+#endif
 
-/* #include "clientlib.h" */
 #include "slrn.h"
 #include "server.h"
 #include "misc.h"
 #include "post.h"
 #include "group.h"
 #include "art.h"
+#include "uudecode.h"
+#include "util.h"
+#include "chmap.h"
 
 #if SLRN_HAS_MIME
 # include "mime.h"
@@ -36,15 +44,16 @@
 char *Slrn_Courtesy_CC_Message = NULL;
 char *Slrn_Save_Posts_File;
 char *Slrn_Last_Message_Id;
-
+char *Slrn_Post_Custom_Headers;
+int Slrn_Reject_Long_Lines = 1;
 
 #ifdef VMS
 # define SYSTEM_OS_NAME		"VMS"
 #else
 # ifdef __os2__
-#  define SYSTEM_OS_NAME	"OS2"
+#  define SYSTEM_OS_NAME	"OS/2"
 # else
-#  ifdef unix
+#  ifdef __unix__
 #   define SYSTEM_OS_NAME       "UNIX"
 #  else
 #   define SYSTEM_OS_NAME	"UNKNOWN"
@@ -53,8 +62,7 @@ char *Slrn_Last_Message_Id;
 #endif
 
 
-#define SLRN_GEN_MSGID
-#ifdef SLRN_GEN_MSGID
+#if SLRN_HAS_GEN_MSGID
 static char *slrn_create_message_id (void)
 {
    unsigned long pid, now;
@@ -120,10 +128,16 @@ void slrn_add_signature (FILE *fp)
    if ((sfp = slrn_open_home_file (Slrn_User_Info.signature, "r", buf, 0)) != NULL)
      {
 	/* Apparantly some RFC suggests the -- \n. */
-        fprintf(fp, "\n\n-- \n");
-        while (fgets (buf, sizeof(buf) - 1, sfp) != NULL)
+        fputs ("\n\n-- \n", fp);
+	
+	/* If signature file already has -- \n, do not add it. */
+	if ((NULL != fgets (buf, sizeof (buf), sfp))
+	    && (0 != strcmp (buf, "-- \n")))
+	  fputs (buf, fp);
+	  
+        while (NULL != fgets (buf, sizeof(buf), sfp))
 	  {
-	     fprintf(fp, "%s", buf);
+	     fputs (buf, fp);
 	  }
         slrn_fclose(sfp);
      }
@@ -143,7 +157,7 @@ static int is_empty_header (char *line)
 }
 
 
-static int slrn_cc_file (char *file, char *to)
+static int slrn_cc_file (char *file, char *to, char *msgid)
 {
 #if defined(VMS) || !SLRN_HAS_PIPING
    return -1;
@@ -230,6 +244,17 @@ static int slrn_cc_file (char *file, char *to)
    rewind (fp);
    linenum = 0;
    
+#ifdef __os2__
+   fprintf (pp, "From: %s@%s (%s)\n",
+            Slrn_User_Info.username, Slrn_User_Info.host,
+            Slrn_User_Info.realname);
+#endif
+   
+   if (msgid != NULL)
+     fprintf (pp, "Message-Id: <slrn%s@%s>\n", 
+	      msgid, Slrn_User_Info.posting_host);
+   
+   
    while ((NULL != fgets (line, sizeof (line) - 1, fp)) && (*line != '\n'))
      {
 	linenum++;
@@ -251,6 +276,13 @@ static int slrn_cc_file (char *file, char *to)
 	else
 	  fputs (line, pp);
      }
+
+#ifdef __os2__
+   /* Add the correct From: line under OS/2 Warp */
+   fprintf (pp, "From: %s@%s (%s)\n",
+	    Slrn_User_Info.username, Slrn_User_Info.host,
+	    ((Slrn_User_Info.realname == NULL) ? "" : Slrn_User_Info.realname));
+#endif
    
 # if SLRN_HAS_MIME
    if (Slrn_Use_Mime) slrn_mime_add_headers (pp);
@@ -275,9 +307,6 @@ static int slrn_cc_file (char *file, char *to)
 #endif
 }
 
-/* This function needs work-- in fact, I ought to move this to a separate
- * file for this purpose.
- */
 static int check_file_for_posting (char *file)
 {
    char line[1024], *the_line;
@@ -381,28 +410,47 @@ static int check_file_for_posting (char *file)
    /* Now body.  Check for non-quoted lines. */
    if (err == NULL) 
      {
-	unsigned int qlen = strlen (Slrn_Quote_String);
+	char *qs = Slrn_Quote_String;
+	unsigned int qlen;
+	
+	if (qs == NULL) qs = ">";
+	qlen = strlen (qs);
 	
 	err = "Your message does not appear to have any unquoted text.";
+	the_line = NULL;
 	while (NULL != fgets (line, sizeof (line), fp))
 	  {
 	     num++;
-	     if (!strncmp (line, Slrn_Quote_String, qlen))
+	     if (!strncmp (line, qs, qlen))
 	       continue;
 	     
 	     colon = slrn_skip_whitespace (line);
 	     if (*colon == 0) continue;
 	     
 	     err = NULL;
-	     break;
+	     if (Slrn_Reject_Long_Lines == 0)
+	       break;
+	     
+	     if (strlen (line) > 81)   /* allow \n to slip */
+	       {
+		  err = "Line contains more than 80 characters.  You need to wrap it.";
+		  the_line = line;
+		  break;
+	       }
 	  }
-	the_line = NULL;
      }
    
    fclose (fp);
 
    if (err == NULL) return 0;
-
+   
+   if (Slrn_Batch)
+     {
+	slrn_error ("Message is not acceptable.");
+	if (err != NULL) slrn_error ("Reason: %s", err);
+	return -1;
+     }
+	  
    Slrn_Full_Screen_Update = 1;
 
    slrn_set_color (0);
@@ -432,7 +480,7 @@ static int check_file_for_posting (char *file)
    rsp |= 0x20;
    if (rsp == 'y')
      {
-	if (slrn_edit_file (file, num) < 0) return -1;
+	if (slrn_edit_file (Slrn_Editor_Post, file, num) < 0) return -1;
 	goto try_again;
      }
    return -1;
@@ -445,20 +493,21 @@ int slrn_post_file (char *file, char *to)
    FILE *fp;
    int rsp;
    int perform_cc;
-#ifdef SLRN_GEN_MSGID
-   char *msgid;
+   int status;
+   char *msgid = NULL;
+#if SLRN_HAS_GEN_MSGID
    int has_messageid = 0;
 #endif
    
    try_again:
    perform_cc = 0;
-   while (1)
+   if (Slrn_Batch == 0) while (1)
      {
 	rsp = slrn_get_response ("yYnNeE", "Post the message? Y-es, N-o, E-dit");
 	rsp |= 0x20;
 	if (rsp == 'n') return -1;
 	if (rsp == 'y') break;
-	if (slrn_edit_file (file, 1) < 0) 
+	if (slrn_edit_file (Slrn_Editor_Post, file, 1) < 0) 
 	  {
 	     slrn_error ("User abort.");
 	     return -1;
@@ -467,7 +516,7 @@ int slrn_post_file (char *file, char *to)
    
    if (-1 == check_file_for_posting (file)) return -1;
    
-   slrn_message ("Posting...");  slrn_smg_refresh ();
+   slrn_message_now ("Posting ...");
    
    if ((Slrn_Save_Posts_File != NULL) && (*Slrn_Save_Posts_File))
      {
@@ -504,11 +553,13 @@ int slrn_post_file (char *file, char *to)
 	     
 	     fputs (line, outfp);
 	  }
-	fputs ("\n\n", outfp);
+	fputs ("\n\n", outfp);	       /* separator */
 	slrn_fclose (infp);
 	slrn_fclose (outfp);
      }
    
+   slrn_chmap_fix_file (file);
+   
    if ((fp = fopen (file, "r")) == NULL)
      {
 	slrn_error ("File not found: %s--- message not posted.", file);
@@ -519,15 +570,22 @@ int slrn_post_file (char *file, char *to)
    if (Slrn_Use_Mime)
      slrn_mime_scan_file (fp);
 #endif
-   
-   if (Slrn_Post_Obj->start () < 0) return -1;
+
+   /* slrn_set_suspension (1); */
+   status = Slrn_Post_Obj->po_start ();
+   if (status != CONT_POST)
+     {
+	if (status != -1)
+	  slrn_error ("Posting not allowed.");
+	return -1;
+     }
    
    if (Slrn_User_Info.username != NULL)
-     Slrn_Post_Obj->printf ("Path: %s\r\n", Slrn_User_Info.username);
+     Slrn_Post_Obj->po_printf ("Path: %s\n", Slrn_User_Info.username);
    
-   Slrn_Post_Obj->printf ("From: %s@%s (%s)\r\n", Slrn_User_Info.username, Slrn_User_Info.host, Slrn_User_Info.realname);
+   Slrn_Post_Obj->po_printf ("From: %s@%s (%s)\n", Slrn_User_Info.username, Slrn_User_Info.host, Slrn_User_Info.realname);
    /* if (Slrn_User_Info.posting_host != NULL)
-     * Slrn_Post_Obj->printf ("X-Posting-Host: %s\r\n", Slrn_User_Info.posting_host); */
+     * Slrn_Post_Obj->po_printf ("X-Posting-Host: %s\n", Slrn_User_Info.posting_host); */
 
    linep = line + 1;
    header = 1;
@@ -543,21 +601,21 @@ int slrn_post_file (char *file, char *to)
 	     while ((*white == ' ') || (*white == '\t')) white++;
 	     if (*white == '\n')
 	       {
-#ifdef SLRN_GEN_MSGID
+#if SLRN_HAS_GEN_MSGID
 		  if (has_messageid == 0)
 		    {
 		       Slrn_Last_Message_Id = msgid = slrn_create_message_id ();
 		       if (msgid != NULL)
 			 {
-			    Slrn_Post_Obj->printf ("Message-Id: <slrn%s@%s>\r\n", msgid, Slrn_User_Info.posting_host);
+			    Slrn_Post_Obj->po_printf ("Message-Id: <slrn%s@%s>\n", msgid, Slrn_User_Info.posting_host);
 			 }
 		    }
 #endif
 #if SLRN_HAS_MIME
 		  if (Slrn_Use_Mime)
-		    slrn_mime_add_headers (0);   /* 0 --> Slrn_Post_Obj->puts */
+		    slrn_mime_add_headers (0);   /* 0 --> Slrn_Post_Obj->po_puts */
 #endif
-		  Slrn_Post_Obj->printf ("X-Newsreader: slrn (%s %s)\r\n\r\n", Slrn_Version, 
+		  Slrn_Post_Obj->po_printf ("X-Newsreader: slrn (%s %s)\n\n", Slrn_Version, 
 			       SYSTEM_OS_NAME);
 		  header = 0;
 #if SLRN_HAS_MIME
@@ -576,7 +634,7 @@ int slrn_post_file (char *file, char *to)
 		  continue;
 	       }
 	     if (is_empty_header (linep)) continue;
-#ifdef SLRN_GEN_MSGID
+#if SLRN_HAS_GEN_MSGID
 	     if (!slrn_case_strncmp ((unsigned char *)"Message-Id: ",
 				     (unsigned char *)linep, 12))
 	       has_messageid = 1;
@@ -587,8 +645,10 @@ int slrn_post_file (char *file, char *to)
 	       slrn_mime_header_encode (linep, sizeof (line) - 1);
 #endif
 	  }
-	
-	
+
+	/* Since the header may not have ended in a \n, make sure the 
+	 * other lines do not either.  Later, the \n will be added.
+	 */
 	if (linep[len - 1] == '\n')
 	  {
 	     len--;
@@ -601,29 +661,34 @@ int slrn_post_file (char *file, char *to)
 	     *linep = '.';
 	  }
 	
-	Slrn_Post_Obj->puts (linep);
-	Slrn_Post_Obj->puts ("\r\n");
+	Slrn_Post_Obj->po_puts (linep);
+	Slrn_Post_Obj->po_puts ("\n");
 	
 	linep = line + 1;
      }
    slrn_fclose (fp);
    
-   if (Slrn_Post_Obj->end () == 0) slrn_message ("Posting...done.");
+   if (0 == Slrn_Post_Obj->po_end ())
+     slrn_message ("Posting...done.");
    else
      {
+	if (Slrn_Batch) return -1;
+
+	/* slrn_set_suspension (0); */
 	slrn_smg_refresh ();
 	sleep (2);
 	slrn_clear_message (); SLang_Error = 0;
-	rsp = slrn_get_response ("RrEeCc", "Select one: R-epost, E-dit, C-ancel");
+	rsp = slrn_get_response ("RrEeCc\007", "Select one: R-epost, E-dit, C-ancel");
+	if (rsp == 7) return -1;
 	rsp |= 0x20;
 	if (rsp == 'c') return -1;
-	if ((rsp == 'e') && (slrn_edit_file (file, 1) < 0)) return -1;
+	if ((rsp == 'e') && (slrn_edit_file (Slrn_Editor_Post, file, 1) < 0)) return -1;
 	goto try_again;
      }
    
    if (perform_cc)
      {
-	slrn_cc_file (file, to);
+	slrn_cc_file (file, to, msgid);
      }
    return 0;
 }
@@ -664,12 +729,14 @@ int slrn_post (char *newsgroup, char *subj, char *dist)
    
    if (dist != NULL) fprintf (fp, "Distribution: %s\n", dist);
    
+   header_lines += slrn_add_custom_headers (fp, Slrn_Post_Custom_Headers, NULL);
+
    fputs ("\n", fp);
    
    slrn_add_signature (fp);
    slrn_fclose (fp);
    
-   if (slrn_edit_file (file, header_lines) >= 0)
+   if (slrn_edit_file (Slrn_Editor_Post, file, header_lines) >= 0)
      {
 	ret = slrn_post_file (file, NULL);
      }
@@ -681,7 +748,40 @@ int slrn_post (char *newsgroup, char *subj, char *dist)
 
 
 
+int slrn_add_custom_headers (FILE *fp, char *headers, int (*write_fun)(char *, FILE *))
+{
+   int n;
+   char *s, *s1, ch, last_ch;
+   
+   if (headers == NULL) return 0;
+   
+   s = slrn_skip_whitespace (headers);
+   if (*s == 0) return 0;
+   
+   s1 = s;
+   n = 0;
+   last_ch = 0;
+   while ((ch = *s1) != 0)
+     {
+	if (ch == '\n') n++;
+	last_ch = ch;
+	s1++;
+     }
 
+   if (write_fun != NULL)
+     (*write_fun)(s, fp);
+   else fputs (s, fp);
+   
+   if (last_ch != '\n')
+     {
+	fputc ('\n', fp);
+	n++;
+     }
+   
+   return n;
+}
+
+   
 
 
 
