@@ -1,30 +1,44 @@
 #include "config.h"
-#include "features.h"
+
+#ifndef SLRNPULL_CODE
+# include "slrnfeat.h"
+#endif
 
 #include <stdio.h>
 #include <string.h>
 #include <sys/types.h>
 #include <time.h>
+
+#ifdef HAVE_STDLIB_H
+# include <stdlib.h>
+#endif
+
 #include <slang.h>
+#include "jdmacros.h"
 
+#ifndef SLRNPULL_CODE
+# include "slrn.h"
+# include "group.h"
+# include "misc.h"
+# include "server.h"
+# include "hash.h"
+# include "util.h"
+#endif
 
-#include "slrn.h"
-#include "group.h"
 #include "art.h"
+#include "xover.h"
 #include "score.h"
-#include "misc.h"
-#include "server.h"
 
-/* This file will implement article scoring for slrn.  Basically the idea is
- * that the user will provide a set of regular expressions that will act on
- * the header of an article and return an integer or 'score'.  If this score
- * is less than zero, the article will be killed.  If it is a positive number
- * greater than some user defined value, the article will be flagged as
- * interesting.  If the score is zero, the article is not killed and it is not
- * flagged as interesting either.
+/* The score file will implement article scoring for slrn.  Basically the idea 
+ * is that the user will provide a set of regular expressions that
+ * will act on the header of an article and return an integer or
+ * 'score'. If this score is less than zero, the article will be
+ * killed.  If it is a positive number greater than some user defined
+ * value, the article will be flagged as interesting.  If the score
+ * is zero, the article is not killed and it is not flagged as
+ * interesting either.
  */
 
-char *Slrn_Score_File;
 int Slrn_Perform_Scoring = SLRN_XOVER_SCORING | SLRN_EXPENSIVE_SCORING;
 
 /* These two structures are pseudo-score types containing no compiled
@@ -42,6 +56,7 @@ typedef struct PScore_Regexp_Type
 #define SCORE_REFERENCES 5
 #define SCORE_LINES 6
 #define SCORE_MESSAGE_ID 7
+#define SCORE_DATE 8
    /* generic requires extra server interaction */
 #define SCORE_GENERIC 16
    unsigned int flags;
@@ -71,7 +86,7 @@ PScore_Type;
 typedef struct
 {
    SLRegexp_Type regexp;
-   unsigned char buf[256];	       /* for compiled pattern */
+   unsigned char buf[512];	       /* for compiled pattern */
 }
 Our_SLRegexp_Type;
 
@@ -115,6 +130,7 @@ Group_Score_Name_Type;
 
 typedef struct Group_Score_Type
 {
+   unsigned int gst_not_flag;
    Group_Score_Name_Type gsnt;
    PScore_Type *pst;
    struct Group_Score_Type *next;
@@ -131,16 +147,31 @@ int slrn_score_header (Slrn_Header_Type *h, char *newsgroup)
 {
    Score_Type *st;
    int score = 0;
-   char *s = NULL;
+   char *s;
    int ival = 0;
    
+#ifndef SLRNPULL_CODE
+#if SLRN_HAS_MSGID_CACHE
+   s = slrn_is_msgid_cached (h->msgid, newsgroup, 1);
+   if (s != NULL)
+     {
+	/* Kill it if this wasn't the newsgroup where we saw it */
+	if (strcmp (s, newsgroup))
+	  return Slrn_Kill_Score_Max;
+     }
+#endif
+#endif				       /* NOT SLRNPULL_CODE */
    
+   s = NULL;
+
    st = Score_Root;
    while (st != NULL)
      {
 	unsigned int len;
 	Score_Regexp_Type *srt = &st->regexp_list;
+#ifndef SLRNPULL_CODE
 	char buf[1024];
+#endif
 	int or_type = st->pscore->flags & SCORE_IS_OR_TYPE;
 	
 	while (srt != NULL)
@@ -159,14 +190,16 @@ int slrn_score_header (Slrn_Header_Type *h, char *newsgroup)
 		  s = h->from;
 		  break;
 		  
+		case SCORE_DATE:
+		  s = h->date;
+		  break;
+
 		case SCORE_MESSAGE_ID:
 		  s = h->msgid;
 		  break;
 		  
 		case SCORE_XREF:
 		  s = h->xref;
-		  /* Note: the xref string includes Xref: */
-		  if (s != NULL) s += 6;
 		  break;
 		  
 		case SCORE_REFERENCES:
@@ -178,11 +211,15 @@ int slrn_score_header (Slrn_Header_Type *h, char *newsgroup)
 		  break;
 		  
 		case SCORE_GENERIC:
+#ifdef SLRNPULL_CODE
+		  s = slrn_get_extra_xover_header (srt->generic_keyword);
+#else
+		  
 		  /* Yuk.  This will be slow! */
 		  if (Slrn_Score_After_XOver)
 		    {
 		       if (((Slrn_Perform_Scoring & SLRN_EXPENSIVE_SCORING) == 0)
-			   || (-1 == Slrn_Server_Obj->xhdr_command (srt->generic_keyword,
+			   || (-1 == Slrn_Server_Obj->sv_xhdr_command (srt->generic_keyword,
 							h->number, buf, sizeof(buf))))
 			 s = NULL;
 		       else s = buf;
@@ -190,9 +227,10 @@ int slrn_score_header (Slrn_Header_Type *h, char *newsgroup)
 		  else
 		    {
 		       if (Slrn_Perform_Scoring & SLRN_EXPENSIVE_SCORING)
-			 s = Slrn_Server_Obj->get_extra_xover_header (srt->generic_keyword);
+			 s = slrn_get_extra_xover_header (srt->generic_keyword);
 		       else s = NULL;
 		    }
+#endif
 		  break;
 		  
 		default:
@@ -211,7 +249,7 @@ int slrn_score_header (Slrn_Header_Type *h, char *newsgroup)
 	       {
 		  SLsearch_Type *se = &srt->search.se;
 		  
-		  if ((len < se->key_len)
+		  if ((len < (unsigned int) se->key_len)
 		      || (NULL == SLsearch ((unsigned char *) s,
 					    (unsigned char *) s + len,
 					    se)))
@@ -311,11 +349,9 @@ static int chain_group_regexp (PScore_Type *pst, int *generic)
    
    while (pst != NULL)
      {
-	if (NULL == (st = (Score_Type *) SLMALLOC (sizeof (Score_Type))))
-	  {
-	     return -1;
-	  }
-	memset ((char *) st, 0, sizeof (Score_Type));
+	st = (Score_Type *) slrn_malloc (sizeof (Score_Type), 1, 0);
+	if (st == NULL)
+	  return -1;
 	
 	if (Score_Root == NULL)
 	  {
@@ -368,12 +404,10 @@ static int chain_group_regexp (PScore_Type *pst, int *generic)
 	     psrt = psrt->next;
 	     if (psrt == NULL) break;
 	     
-	     if (NULL == (srt->next = (Score_Regexp_Type *) SLMALLOC (sizeof (Score_Regexp_Type))))
-	       {
-		  return -1;
-	       }
-	     srt = srt->next;
-	     memset ((char *) srt, 0, sizeof (Score_Regexp_Type));
+	     srt->next = (Score_Regexp_Type *) slrn_malloc (sizeof (Score_Regexp_Type),
+							    1, 0);
+	     if (NULL == (srt = srt->next))
+	       return -1;
 	  }
 	pst = pst->next;
      }
@@ -419,15 +453,28 @@ int slrn_open_score (char *group_name)
      {
 	Group_Score_Name_Type *gsnt;
 	SLRegexp_Type *re;
+	unsigned int gst_not_flag;
 	
 	gsnt = &gsc->gsnt;
+	gst_not_flag = gsc->gst_not_flag;
 	
 	while (gsnt != NULL)
 	  {
+	     int match;
+	     
 	     re = &gsnt->group_regexp;
 	
-	     if ((re->min_length <= n)
-		 && (NULL != SLang_regexp_match ((unsigned char *) group_name, n, re)))
+	     match = ((re->min_length <= n)
+		      && (NULL != SLang_regexp_match ((unsigned char *) group_name, n, re)));
+	     
+	     /* Note: the following could be replaced by simply
+	      *  match = (match ^ gst_not_flag);
+	      * however, I may have more uses for this flag in the future.
+	      */
+	     match = ((match && (gst_not_flag == 0))
+		      || ((match == 0) && gst_not_flag));
+	       
+	     if (match)
 	       {
 		  if (-1 == chain_group_regexp (gsc->pst, &generic))
 		    {
@@ -448,19 +495,24 @@ int slrn_open_score (char *group_name)
    if ((generic == 0)
        || (0 == (Slrn_Perform_Scoring & SLRN_EXPENSIVE_SCORING)))
      return 1;
-   
-   if ((generic >= 1) || !Slrn_Server_Obj->has_xover || !Slrn_Server_Obj->has_cmd ("XHDR"))
+
+#ifndef SLRNPULL_CODE
+   if ((generic >= 1) || !Slrn_Server_Obj->sv_has_xover || !Slrn_Server_Obj->sv_has_cmd ("XHDR"))
      {
 	Slrn_Score_After_XOver = 0;
-	Slrn_Server_Obj->open_suspend_xover ();
+	slrn_open_suspend_xover ();
      }
+#endif
+
    return 1;
 }
 
 void slrn_close_score (void)
 {
    free_group_chain ();
-   Slrn_Server_Obj->close_suspend_xover ();
+#ifndef SLRNPULL_CODE
+   slrn_close_suspend_xover ();
+#endif
 }
 
 
@@ -482,17 +534,10 @@ static int add_group_regexp (PScore_Type *pst,
    len = (unsigned int) (slrn_trim_string ((char *) str) - (char *) str);
    if (0 == len) return -1;
    
-   if (NULL == (psrt = (PScore_Regexp_Type *) SLMALLOC (sizeof (PScore_Regexp_Type))))
-     slrn_exit_error ("Memory Allocation Failure.");
-   
-   memset ((char *) psrt, 0, sizeof (PScore_Regexp_Type));
+   psrt = (PScore_Regexp_Type *) slrn_safe_malloc (sizeof (PScore_Regexp_Type));
    
    if (type != SCORE_LINES)
-     {
-	if (NULL == (psrt->ireg.regexp_str = (unsigned char *) SLMALLOC (len + 1)))
-	  slrn_exit_error ("Memory Allocation Failure.");
-	strcpy ((char *)psrt->ireg.regexp_str, (char *) str);
-     }
+     psrt->ireg.regexp_str = (unsigned char *) slrn_safe_strmalloc ((char *)str);
    else
      {
 	psrt->ireg.ival = atoi((char *)str);
@@ -560,10 +605,7 @@ static int compile_group_names (char *group, Group_Score_Type *gst)
 	  {
 	     Group_Score_Name_Type *gsnt1;
 	     
-	     gsnt1 = (Group_Score_Name_Type *) SLMALLOC (sizeof (Group_Score_Name_Type));
-	     if (gsnt1 == NULL) 
-	       slrn_exit_error ("Memory allocation failure.");
-	     memset ((char *) gsnt1, 0, sizeof (Group_Score_Name_Type));
+	     gsnt1 = (Group_Score_Name_Type *) slrn_safe_malloc (sizeof (Group_Score_Name_Type));
 	     
 	     gsnt->next = gsnt1;
 	     gsnt = gsnt1;
@@ -575,7 +617,7 @@ static int compile_group_names (char *group, Group_Score_Type *gst)
    return -1;
 }
 
-static PScore_Type *create_new_score (unsigned char *group, int new_group_flag,
+static PScore_Type *create_new_score (unsigned char *group, int new_group_flag, unsigned int gst_not_flag,
 				      int score, unsigned int pscore_flags)
 {
    PScore_Type *pst;
@@ -588,6 +630,7 @@ static PScore_Type *create_new_score (unsigned char *group, int new_group_flag,
 	while (Group_Score_Next != NULL)
 	  {
 	     if ((Group_Score_Next->gsnt.next == NULL)
+		 && (Group_Score_Next->gst_not_flag == gst_not_flag)
 		 && !strcmp ((char *) group, Group_Score_Next->gsnt.name))
 	       break;
 	     tail = Group_Score_Next;
@@ -596,14 +639,12 @@ static PScore_Type *create_new_score (unsigned char *group, int new_group_flag,
 	
 	if (Group_Score_Next == NULL)
 	  {
-	     gst = (Group_Score_Type *) SLMALLOC (sizeof (Group_Score_Type));
-	     if (gst == NULL)
-	       slrn_exit_error ("Memory allocation failure.");
-	     
-	     memset ((char *)gst, 0, sizeof (Group_Score_Type));
+	     gst = (Group_Score_Type *) slrn_safe_malloc (sizeof (Group_Score_Type));
 	     
 	     if (-1 == compile_group_names ((char *)group, gst))
 	       return NULL;
+	     
+	     gst->gst_not_flag = gst_not_flag;
 
 	     if (Group_Score_Root == NULL)
 	       {
@@ -616,10 +657,8 @@ static PScore_Type *create_new_score (unsigned char *group, int new_group_flag,
      }
    
    /* Now create the PseudoScore type and add it. */
-   if (NULL == (pst = (PScore_Type *) SLMALLOC (sizeof (PScore_Type))))
-     slrn_exit_error ("Memory Allocation Failure.");
-   
-   memset ((char *) pst, 0, sizeof (PScore_Type));
+   pst = (PScore_Type *) slrn_safe_malloc (sizeof (PScore_Type));
+
    pst->score = score;
    pst->flags = pscore_flags;
    
@@ -670,9 +709,8 @@ static void free_group_scores (void)
 	       {
 		  PScore_Regexp_Type *rnext = r->next;
 		  
-		  if (((r->flags & USE_INTEGER) == 0)
-		      && (r->ireg.regexp_str != NULL))
-		    SLFREE (r->ireg.regexp_str);
+		  if ((r->flags & USE_INTEGER) == 0)
+		    slrn_free ((char *) r->ireg.regexp_str);
 		  SLFREE (r);
 		  r = rnext;
 	       }
@@ -720,13 +758,12 @@ static unsigned long get_today (void)
    return yy * 10000 + mm * 100 + dd;
 }
 
-int slrn_read_score_file (char *name)
+int slrn_read_score_file (char *file)
 {
-   char file[256];
    unsigned char group[256];
    char line[1024];
    FILE *fp;
-   int start_new_group = 1, not_flag;
+   int start_new_group = 1, not_flag, gnt_not_flag = 0;
    unsigned int linenum = 0;
    int score = 0;
    PScore_Type *pst = NULL;
@@ -741,7 +778,8 @@ int slrn_read_score_file (char *name)
 	free_group_scores ();
      }
    
-   fp = slrn_open_home_file (name, "r", file, 0);
+   fp = fopen (file, "r");
+   
    if (fp == NULL)
      {
 	Slrn_Apply_Score = 0;
@@ -766,6 +804,15 @@ int slrn_read_score_file (char *name)
 	     gmax = g + sizeof (group);
 	     
 	     lp++;
+	     lp = (unsigned char *) slrn_skip_whitespace ((char *)lp);
+	     
+	     gnt_not_flag = 0;
+	     if (*lp == '~')
+	       {
+		  gnt_not_flag = 1;
+		  lp = (unsigned char *) slrn_skip_whitespace ((char *)lp + 1);
+	       }
+	     
 	     while (((ch = *lp++) != 0)
 		    && (ch != ']') && (g < gmax))
 	       *g++ = ch;
@@ -782,7 +829,7 @@ int slrn_read_score_file (char *name)
 	     continue;
 	  }
 	
-	if (!strncmp ((char *)lp, "Score:", 6))
+	if (!slrn_case_strncmp (lp, (unsigned char *)"Score:", 6))
 	  {
 	     unsigned char *lpp = lp + 6;
 	     pscore_flags = 0;
@@ -804,7 +851,7 @@ int slrn_read_score_file (char *name)
 	
 	if (start_new_score)
 	  {
-	     if (!strncmp ((char *)lp, "Expires:", 8))
+	     if (!slrn_case_strncmp (lp, (unsigned char *) "Expires:", 8))
 	       {
 		  int ret;
 		  ret = has_score_expired (lp + 8, today);
@@ -826,7 +873,8 @@ int slrn_read_score_file (char *name)
 	       }
 	     
 	     
-	     if (NULL == (pst = create_new_score (group, start_new_group, score, pscore_flags)))
+	     if (NULL == (pst = create_new_score (group, start_new_group, gnt_not_flag,
+						  score, pscore_flags)))
 	       {
 		  score_error ("Bad group regular expression.", line, linenum, file);
 		  goto error_return;
@@ -853,19 +901,21 @@ int slrn_read_score_file (char *name)
 	
 	
 	/* Otherwise the line is a kill one */
-	if (!strncmp ((char *)lp, "Subject:", 8))
+	if (!slrn_case_strncmp (lp, (unsigned char *)"Subject:", 8))
 	  add_group_regexp (pst, lp + 7, lp, SCORE_SUBJECT, not_flag);
-	else if (!strncmp ((char *)lp, "From:", 5))
+	else if (!slrn_case_strncmp (lp, (unsigned char *)"From:", 5))
 	  add_group_regexp (pst, lp + 4, lp, SCORE_FROM, not_flag);
-	else if (!strncmp ((char *)lp, "Xref:", 5))
+	else if (!slrn_case_strncmp (lp, (unsigned char *)"Xref:", 5))
 	  add_group_regexp (pst, lp + 4, lp, SCORE_XREF, not_flag);
-	else if (!strncmp ((char *)lp, "Newsgroup:", 10))
+	else if (!slrn_case_strncmp (lp, (unsigned char *)"Newsgroup:", 10))
 	  add_group_regexp (pst, lp + 9, lp, SCORE_NEWSGROUP, not_flag);
-	else if (!strncmp ((char *)lp, "References:", 11))
+	else if (!slrn_case_strncmp (lp, (unsigned char *)"References:", 11))
 	  add_group_regexp (pst, lp + 10, lp, SCORE_REFERENCES, not_flag);
-	else if (!strncmp ((char *)lp, "Lines:", 6))
+	else if (!slrn_case_strncmp (lp, (unsigned char *)"Lines:", 6))
 	  add_group_regexp (pst, lp + 5, lp, SCORE_LINES, not_flag);
-	else if (!strncmp ((char *)lp, "Message-Id:", 11))
+	else if (!slrn_case_strncmp (lp, (unsigned char *)"Date:", 5))
+	  add_group_regexp (pst, lp + 5, lp, SCORE_DATE, not_flag);
+	else if (!slrn_case_strncmp (lp, (unsigned char *)"Message-Id:", 11))
 	  add_group_regexp (pst, lp + 10, lp, SCORE_MESSAGE_ID, not_flag);
 	else
 	  {
@@ -881,140 +931,12 @@ int slrn_read_score_file (char *name)
 	  }
      }
    
-   slrn_fclose (fp);
+   fclose (fp);
    Slrn_Apply_Score = 1;
    return 1;
    
    error_return:
-   slrn_fclose (fp);
+   fclose (fp);
    return -1;
 }
 
-
-int slrn_edit_score (Slrn_Header_Type *h, char *newsgroup)
-{
-   char ch = 'e';
-   int ich;
-   char file[256];
-   char qregexp[512];
-   unsigned int mm = 0, dd = 0, yy = 0;
-   int use_expired = 0;
-   unsigned int linenum = 0;
-   char *q, *ng = newsgroup;
-   int score;
-   FILE *fp;
-   time_t myclock;
-   
-   
-   if (Slrn_Score_File == NULL)
-     {
-	slrn_error ("A Score file has not been specified.");
-	return -1;
-     }
-   
-   if (Slrn_Prefix_Arg_Ptr == NULL)
-     {
-	ch = slrn_get_response ("SsFfRrEeCc", "Pick Score type: S-ubject, F-rom, R-eferences, E-dit, C-ancel");
-	ch |= 0x20;
-	if (ch == 'c') return -1;
-   
-	score = -9999;
-	if (-1 == slrn_read_integer ("Score", &score, &score))
-	  return -1;
-	if (score == 0) return -1;
-	
-	if ('a' == (0x20 | slrn_get_response ("TtaA", "Which newsgroups: T-his group, A-ll groups")))
-	  ng = "*";
-
-	while (1)
-	  {
-	     *qregexp = 0;
-	     if (-1 == slrn_read_input ("Expires (MM/DD/YYYY or DD-MM-YYYY or leave blank):", qregexp, 1))
-	       return -1;
-	     if (*qregexp)
-	       {
-		  if (((3 != sscanf (qregexp, "%u/%u/%u", &mm, &dd, &yy))
-		       && (3 != sscanf (qregexp, "%u-%u-%u", &dd, &mm, &yy)))
-		      || (dd > 31)
-		      || (mm > 12)
-		      || (yy < 1900))
-		    continue;
-		  use_expired = 1;
-		  break;
-	       }
-	     else
-	       {
-		  use_expired = 0;
-		  break;
-	       }
-	  }
-     }
-   
-   if ((NULL == (fp = slrn_open_home_file (Slrn_Score_File, "r+", file, 1)))
-       && (NULL == (fp = slrn_open_home_file (Slrn_Score_File, "w+", file, 1))))
-     {
-	slrn_error ("Unable to open %s", file);
-	return -1;
-     }
-   
-   if (Slrn_Prefix_Arg_Ptr == NULL)
-     {
-	linenum = 1;
-	while (EOF != (ich = getc (fp)))
-	  {
-	     if (ich == '\n')	linenum++;
-	  }
-	
-	myclock = time((time_t *) 0);
-	fprintf (fp, "\n%%Score created by slrn on %s\n[%s]\nScore: %d\n",
-		 (char *) ctime(&myclock), ng, score);
-	
-	if (use_expired)
-	  fprintf (fp, "Expires: %u/%u/%u\n", mm, dd, yy);
-	else fprintf (fp, "%%Expires: \n");
-	
-	q = SLregexp_quote_string (h->subject, qregexp, sizeof (qregexp));
-	if (q == NULL) q = h->subject;
-	fprintf (fp, "%c\tSubject: %s\n",
-		 ((ch == 's') ? ' ' : '%'), q);
-	
-	q = SLregexp_quote_string (h->from, qregexp, sizeof (qregexp));
-	if (q == NULL) q = h->from;
-	fprintf (fp, "%c\tFrom: %s\n",
-		 ((ch == 'f') ? ' ' : '%'), q);
-	
-	q = SLregexp_quote_string (h->msgid, qregexp, sizeof (qregexp));
-	if (q == NULL) q = h->msgid;
-	fprintf (fp, "%c\tReferences: %s\n",
-		 ((ch == 'r') ? ' ' : '%'), q);
-	
-	
-	q = SLregexp_quote_string (h->xref, qregexp, sizeof(qregexp));
-	if (q != NULL)
-	  {
-	     /* The way slrn handles Xref, we only need this: */
-	     fprintf (fp, "%%\t%s\n", q);
-	  }
-	
-	q = SLregexp_quote_string (newsgroup, qregexp, sizeof(qregexp));
-	if (q == NULL) q = newsgroup;
-	fprintf (fp, "%%\tNewsgroup: %s\n", q);
-     }
-   
-   Slrn_Prefix_Arg_Ptr = NULL;
-   slrn_fclose (fp);
-
-   if (ch == 'e')
-     {
-	if (-1 == slrn_edit_file (file, linenum + 1))
-	  {
-	     slrn_error ("Error calling editor.");
-	     return -1;
-	  }
-     }
-   slrn_message ("Reloading score file...");
-   slrn_smg_refresh ();
-   (void) slrn_read_score_file (Slrn_Score_File);
-   slrn_message ("Exit group for changes to take effect.");
-   return 0;
-}
