@@ -3,8 +3,6 @@ extern int slrn_score_header (Slrn_Header_Type *, char *);
 extern int slrn_read_score_file (char *);
 extern void slrn_close_score (void);
 extern int slrn_open_score (char *);
-extern int slrn_edit_score (Slrn_Header_Type *, char *);
-extern char *Slrn_Score_File;
 #define SLRN_XOVER_SCORING 1
 #define SLRN_EXPENSIVE_SCORING 2
 extern int Slrn_Perform_Scoring;
