@@ -4,7 +4,7 @@
  * This file is used to indicate capabilities of the C compiler and
  * operating system.
  * 
- * See also features.h for customization of slrn features.
+ * See also slrnfeat.h for customization of slrn features.
  */
 
 #define USE_DOMAIN_NAME 0
@@ -31,6 +31,10 @@
 #if defined (__os2__)
 # define HAVE_UNISTD_H 1
 # define HAVE_MEMORY_H 1
+# define HAVE_SYS_SOCKET_H 1
+# define HAVE_NETINET_IN_H 1
+# define HAVE_ARPA_INET_H 1
+# define HAVE_DIRENT_H 1
 #endif
 
 /* 
@@ -51,4 +55,5 @@
 
 #define SLRN_POST_ID_NNTP 1
 #define SLRN_POST_ID_INEWS 2
+#define SLRN_POST_ID_PULL 3
 
