@@ -1,18 +1,13 @@
-/*  Copyright (c) 1995 John E. Davis (davis@space.mit.edu)
+/* -*- mode: C; mode: fold; -*- */
+/*  Copyright (c) 1995, 1996 John E. Davis (davis@space.mit.edu)
  *  All rights reserved.
  */
 /* Read startup .slrnrc file */
 
-/* This file contains the following types of lines:
- *    server SERVER-NAME NEWSRC-FILE
- *    setkey KEYMAP FUNCTION KEYSEQUENCE
- *    unsetkey KEYMAP KEYSEQUENCE
- *
- * Lines are considered commented out if they start with a '%' character.
- */
-
 #include "config.h"
-#include "features.h"
+#include "slrnfeat.h"
+
+/*{{{ Include Files */
 
 #include <stdio.h>
 #ifdef HAVE_STDLIB_H
@@ -22,6 +17,7 @@
 #include <string.h>
 #include <slang.h>
 
+#include "jdmacros.h"
 
 #include "slrn.h"
 #include "group.h"
@@ -30,15 +26,26 @@
 #include "post.h"
 #include "startup.h"
 #include "score.h"
+#include "util.h"
 #include "uudecode.h"
 #if SLRN_HAS_MIME
 # include "mime.h"
 #endif
+#if SLRN_HAS_GROUPLENS
+# include "grplens.h"
+#endif
+#if SLRN_HAS_SLANG
+# include "interp.h"
+#endif
 #include "server.h"
+#include "chmap.h"
 
+#ifdef VMS
+# include "vms.h"
+#endif
+/*}}}*/
 
-static SLcmd_Cmd_Table_Type Slrn_Cmd_Table;
-
+/*{{{ Forward Function Declarations */
 
 static int unsetkey_fun (int, SLcmd_Cmd_Table_Type *);
 static int setkey_fun (int, SLcmd_Cmd_Table_Type *);
@@ -47,12 +54,21 @@ static int color_fun (int, SLcmd_Cmd_Table_Type *);
 static int mono_fun (int, SLcmd_Cmd_Table_Type *);
 static int user_data_fun (int, SLcmd_Cmd_Table_Type *);
 static int ignore_quote_fun (int, SLcmd_Cmd_Table_Type *);
-static int set_quote_fun (int, SLcmd_Cmd_Table_Type *);
 static int autobaud_fun (int, SLcmd_Cmd_Table_Type *);
 static int set_variable_fun (int, SLcmd_Cmd_Table_Type *);
 static int nnrp_fun (int, SLcmd_Cmd_Table_Type *);
+static int grouplens_fun (int, SLcmd_Cmd_Table_Type *);
+static int interpret_fun (int, SLcmd_Cmd_Table_Type *);
 
-static SLcmd_Cmd_Type Slrn_Startup_File_Cmds[] =
+/*}}}*/
+/*{{{ Static Global Variables */
+
+static int This_Line_Num;	       /* current line number in startup file */
+static char *This_File;
+static char *This_Line;		       /* line being parsed */
+
+static SLcmd_Cmd_Table_Type Slrn_Cmd_Table;
+static SLcmd_Cmd_Type Slrn_Startup_File_Cmds[] = /*{{{*/
 {
      {unsetkey_fun, "unsetkey", "SS"},
      {setkey_fun, "setkey", "SSS"},
@@ -60,34 +76,46 @@ static SLcmd_Cmd_Type Slrn_Startup_File_Cmds[] =
      {color_fun, "color", "SSS"},
      {mono_fun, "mono", "SSsss"},
      {set_variable_fun, "set", "SG"},
+     {user_data_fun, "username", "S"},
+     {user_data_fun, "hostname", "S"},
+     {nnrp_fun, "nnrpaccess", "SSS" },
+#define SLRN_MAX_QUOTE_REGEXP 5
+     {ignore_quote_fun, "ignore_quotes", "Sssss"},
+     {autobaud_fun, "autobaud", ""},
+     {grouplens_fun, "grouplens_add", "S"},
+     {interpret_fun, "interpret", "S"},
+     {user_data_fun, "scorefile", "S"},
+   
+   /* The following are considered obsolete */
      {user_data_fun, "replyto", "S"},
      {user_data_fun, "organization", "S"},
      {user_data_fun, "signature", "S"},
-     {user_data_fun, "hostname", "S"},
      {user_data_fun, "realname", "S"},
      {user_data_fun, "followup", "S"},
-     {user_data_fun, "username", "S"},
-     {user_data_fun, "scorefile", "S"},
      {user_data_fun, "cc_followup_string", "S"},
+     {user_data_fun, "quote_string", "S"},
 #if SLRN_HAS_DECODE
      {user_data_fun, "decode_directory", "S"},
 #endif
      {user_data_fun, "editor_command", "S"},
-     {nnrp_fun, "nnrpaccess", "SSS" },
-#define SLRN_MAX_QUOTE_REGEXP 5
-     {ignore_quote_fun, "ignore_quotes", "Sssss"},
-     {set_quote_fun, "quote_string", "S"},
-     {autobaud_fun, "autobaud", ""},
      {NULL, "", ""}
 };
 
+/*}}}*/
+
+/*}}}*/
+/*{{{ Public Global Variables */
+
 SLRegexp_Type *Slrn_Ignore_Quote_Regexp [SLRN_MAX_QUOTE_REGEXP + 1];
+int Slrn_Autobaud = 0;
+char *Slrn_Score_File;
+int Slrn_Scroll_By_Page;
 
-static int This_Line_Num;		       /* current line number in startup file */
-static char *This_File;
-static char *This_Line;		       /* line being parsed */
+/*}}}*/
 
-static void exit_malloc_error (void)
+/*{{{ Utility Functions */
+
+static void exit_malloc_error (void) /*{{{*/
 {
    if (This_File == NULL)
      slrn_exit_error ("Memory Allocation Failure");
@@ -96,7 +124,9 @@ static void exit_malloc_error (void)
 		    This_File, This_Line_Num, This_Line);
 }
 
-static char *safe_malloc (unsigned int n)
+/*}}}*/
+
+static char *safe_malloc (unsigned int n) /*{{{*/
 {
    char *s;
    s = (char *) SLMALLOC (n);
@@ -104,24 +134,42 @@ static char *safe_malloc (unsigned int n)
    return s;
 }
 
+/*}}}*/
 
-static void exit_unknown_object (void)
+
+static void exit_unknown_object (void) /*{{{*/
 {
    slrn_exit_error ("%s: Error encountered processing line %d\n%s",
 		    This_File, This_Line_Num, This_Line);
 }
 
+/*}}}*/
 
-static int setkey_fun (int argc, SLcmd_Cmd_Table_Type *table)
+static void issue_obsolete_message (void) /*{{{*/
+{
+   slrn_message ("%s: Command is obsolete on line %d:\n%s",
+		 This_File, This_Line_Num, This_Line);
+   slrn_message ("The new usage is:\nset %s\n", This_Line);
+}
+
+/*}}}*/
+
+   
+/*}}}*/
+/*{{{ Set/Unset Key Functions */
+
+static int setkey_fun (int argc, SLcmd_Cmd_Table_Type *table) /*{{{*/
 {
    char *map = table->string_args[1];
    char *fun = table->string_args[2];
    char *key = table->string_args[3];
-   
    SLKeyMap_List_Type *kmap = NULL;
    
+   (void) argc;
+
    if (!strcmp (map, "group")) kmap = Slrn_Group_Keymap;
    else if (!strcmp (map, "article")) kmap = Slrn_Article_Keymap;
+   else if (!strcmp (map, "readline")) kmap = Slrn_RLine_Keymap;
    else slrn_exit_error ("%s: line %d:\n%sNo such keymap: %s", This_File, This_Line_Num, This_Line, map);
    
    if (SLang_define_key (key, fun, kmap) != 0)
@@ -131,20 +179,16 @@ static int setkey_fun (int argc, SLcmd_Cmd_Table_Type *table)
    return 0;
 }
 
-int Slrn_Autobaud = 0;
-static int autobaud_fun (int argc, SLcmd_Cmd_Table_Type *table)
-{
-   Slrn_Autobaud = 1;
-   return 0;
-}
+/*}}}*/
 
-
-static int unsetkey_fun (int argc, SLcmd_Cmd_Table_Type *table)
+static int unsetkey_fun (int argc, SLcmd_Cmd_Table_Type *table) /*{{{*/
 {
    char *map = table->string_args[1];
    char *key = table->string_args[2];
    SLKeyMap_List_Type *kmap = NULL;
    
+   (void) argc;
+   
    if (!strcmp (map, "group")) kmap = Slrn_Group_Keymap;
    else if (!strcmp (map, "article")) kmap = Slrn_Article_Keymap;
    else slrn_exit_error ("%s: line %d:\n%sNo such keymap: %s",
@@ -154,8 +198,20 @@ static int unsetkey_fun (int argc, SLcmd_Cmd_Table_Type *table)
    return 0;
 }
 
+/*}}}*/
 
-static SLRegexp_Type *compile_quote_regexp (char *str)
+/*}}}*/
+
+static int autobaud_fun (int argc, SLcmd_Cmd_Table_Type *table) /*{{{*/
+{
+   (void) argc; (void) table;
+   Slrn_Autobaud = 1;
+   return 0;
+}
+
+/*}}}*/
+
+static SLRegexp_Type *compile_quote_regexp (char *str) /*{{{*/
 {
    unsigned char *compiled_pattern_buf;
    SLRegexp_Type *r;
@@ -177,7 +233,8 @@ static SLRegexp_Type *compile_quote_regexp (char *str)
    return r;
 }
 
-static int ignore_quote_fun (int argc, SLcmd_Cmd_Table_Type *table)
+/*}}}*/
+static int ignore_quote_fun (int argc, SLcmd_Cmd_Table_Type *table) /*{{{*/
 {
    unsigned int i;
    SLRegexp_Type *r;
@@ -193,14 +250,13 @@ static int ignore_quote_fun (int argc, SLcmd_Cmd_Table_Type *table)
 	r = Slrn_Ignore_Quote_Regexp[i];
 	if (r != NULL)
 	  {
-	     if (r->buf != NULL)
-	       SLFREE (r->buf);
+	     slrn_free ((char *) r->buf);
 	     SLFREE (r);
 	     Slrn_Ignore_Quote_Regexp [i] = NULL;
 	  }
      }
 	
-   for (i = 1; i < argc; i++)
+   for (i = 1; i < (unsigned int) argc; i++)
      {
 	Slrn_Ignore_Quote_Regexp[i - 1] 
 	  = compile_quote_regexp (table->string_args[i]);
@@ -208,34 +264,53 @@ static int ignore_quote_fun (int argc, SLcmd_Cmd_Table_Type *table)
    return 0;
 }
 
-static int set_quote_fun (int argc, SLcmd_Cmd_Table_Type *table)
+/*}}}*/
+
+static int grouplens_fun (int argc, SLcmd_Cmd_Table_Type *table) /*{{{*/
 {
-   static char quote[32];
-   
-   strncpy (quote, table->string_args[1], 31);
-   quote[31] = 0;
-   Slrn_Quote_String = quote;
+   (void) argc;
+#if SLRN_HAS_GROUPLENS
+   (void) slrn_grouplens_add_group (table->string_args[1]);
+#else 
+   (void) table;
+#endif
    return 0;
 }
 
-typedef struct
+/*}}}*/
+
+/*{{{ Setting/Getting Variable Functions */
+
+typedef struct /*{{{*/
 {
    char *what;
    int *valuep;
 }
+
+/*}}}*/
 Set_Int_Type;
 
-static Set_Int_Type Int_Things_To_Set [] =
+static Set_Int_Type Int_Things_To_Set [] = /*{{{*/
 {
+     {"new_subject_breaks_threads",	&Slrn_New_Subject_Breaks_Threads},
+     {"scroll_by_page", &Slrn_Scroll_By_Page},
+     {"use_color", &SLtt_Use_Ansi_Colors},
      {"ignore_signature", &Slrn_Sig_Is_End_Of_Article},
+     {"reject_long_lines", &Slrn_Reject_Long_Lines},
 #if SLRN_HAS_NNTP_SUPPORT
      {"query_reconnect", &Slrn_Query_Reconnect},
 #else
      {"query_reconnect", NULL},
+#endif
+#if SLRN_HAS_SORT_BY_SCORE
+     {"display_score", &Slrn_Display_Score},
+#else
+     {"display_score", NULL},
 #endif
      {"use_xgtitle", &Slrn_Use_Xgtitle},
      {"show_article", &Slrn_Startup_With_Article},
      {"author_display", &Slrn_Show_Author},
+     {"display_author_realname", &Slrn_Show_Author_Realname},
      {"show_descriptions", &Slrn_Group_Display_Descriptions},
      {"no_backups", &Slrn_No_Backups},
      {"beep", &SLtt_Ignore_Beep},
@@ -253,6 +328,9 @@ static Set_Int_Type Int_Things_To_Set [] =
      {"use_metamail", &Slrn_Use_Meta_Mail},
      {"group_dsc_start_column", &Slrn_Group_Description_Column},
      {"lines_per_update", &Slrn_Reads_Per_Update},
+     {"min_high_score", &Slrn_High_Score_Min},
+     {"max_low_score", &Slrn_Low_Score_Max},
+     {"kill_score", &Slrn_Kill_Score_Max},
 #ifndef __os2__
      {"use_blink", &SLtt_Blink_Mode},
 #endif
@@ -263,7 +341,9 @@ static Set_Int_Type Int_Things_To_Set [] =
      {"use_header_numbers", &Slrn_Use_Header_Numbers},
 #if SLRN_HAS_SPOILERS
      {"spoiler_char", &Slrn_Spoiler_Char},
+     {"spoiler_display_mode", &Slrn_Spoiler_Display_Mode},
 #else
+     {"spoiler_start_new_page", NULL},
      {"spoiler_char", NULL},
 #endif
 #if SLRN_HAS_MIME
@@ -271,29 +351,76 @@ static Set_Int_Type Int_Things_To_Set [] =
 #else
      {"use_mime", NULL},
 #endif
+#if SLRN_HAS_GROUPLENS
+     {"use_grouplens", &Slrn_Use_Group_Lens},
+     {"grouplens_port", &Slrn_GroupLens_Port},
+#else
+     {"use_grouplens", NULL},
+#endif
 #if 0
 #if SLRN_HAS_INEWS_SUPPORT
      {"use_inews", &Slrn_Use_Inews},
 #else
      {"use_inews", NULL},
 #endif
+#endif
+#if SLRN_HAS_PULL_SUPPORT
+     {"use_slrnpull", &Slrn_Use_Pull_Post},
+#else
+     {"use_slrnpull", NULL},
+#endif
+#if SLRN_HAS_TILDE_FEATURE
+     {"use_tilde", &Slrn_Use_Tildes},
+#else
+     {"use_tilde", NULL},
 #endif
      {NULL, NULL}
 };
 
-typedef struct
+/*}}}*/
+
+typedef struct /*{{{*/
 {
    char *what;
    char **svaluep;
 }
+
+/*}}}*/
 Set_String_Type;
 
-static Set_String_Type String_Things_To_Set [] =
+static Set_String_Type String_Things_To_Set [] = /*{{{*/
 {
+     {"art_help_line", &Slrn_Art_Help_Line},
+     {"header_help_line", &Slrn_Header_Help_Line},
+     {"group_help_line", &Slrn_Group_Help_Line},
+     {"quote_string", &Slrn_Quote_String},
+     {"realname", &Slrn_User_Info.realname},
+     {"replyto", &Slrn_User_Info.replyto},
+     {"organization", &Slrn_User_Info.org},
+     {"followup", &Slrn_User_Info.followup_string},
+     {"cc_followup_string", &Slrn_Courtesy_CC_Message},
+#if SLRN_HAS_DECODE
+     {"decode_directory", &Slrn_Decode_Directory},
+#endif
+     {"editor_command", &Slrn_Editor},
+     {"post_editor_command", &Slrn_Editor_Post},
+     {"score_editor_command", &Slrn_Editor_Score},
+     {"mail_editor_command", &Slrn_Editor_Mail},
      {"non_Xbrowser", &Slrn_NonX_Browser},
      {"Xbrowser", &Slrn_X_Browser},
      {"save_posts", &Slrn_Save_Posts_File},
      {"save_directory", &Slrn_Save_Directory},
+     {"signature", &Slrn_User_Info.signature},
+     {"custom_headers", &Slrn_Post_Custom_Headers},
+     {"followup_custom_headers", &Slrn_Followup_Custom_Headers},
+     {"reply_custom_headers", &Slrn_Reply_Custom_Headers},
+#if SLRN_HAS_GROUPLENS
+     {"grouplens_pseudoname", &Slrn_GroupLens_Pseudoname},
+     {"grouplens_host", &Slrn_GroupLens_Host},
+#else
+     {"grouplens_pseudoname", NULL},
+     {"grouplens_host", NULL},
+#endif
      {"decode_directory", 
 #if SLRN_HAS_DECODE
 	  &Slrn_Decode_Directory
@@ -312,8 +439,15 @@ static Set_String_Type String_Things_To_Set [] =
      
 #if SLRN_HAS_MIME
      {"mime_charset", &Slrn_Mime_Display_Charset},
+     {"metamail_command", &Slrn_MetaMail_Cmd},
 #else
      {"mime_charset", NULL},
+     {"metamail_command", NULL},
+#endif
+#if SLRN_HAS_CHARACTER_MAP
+     {"charset", &Slrn_Charset},
+#else
+     {"charset", NULL},
 #endif
 #ifndef VMS
      {"sendmail_command", &Slrn_SendMail_Command},
@@ -367,127 +501,177 @@ static Set_String_Type String_Things_To_Set [] =
      NULL
 #endif
      },
-   
+     {"macro_directory",
+#if SLRN_HAS_SLANG
+	&Slrn_Macro_Dir
+#else
+	  NULL
+#endif
+     },
+
      {NULL, NULL}
 };
 
-static int set_variable_fun (int argc, SLcmd_Cmd_Table_Type *table)
+/*}}}*/
+
+int slrn_set_string_variable (char *name, char *value) /*{{{*/
 {
-   char *what = table->string_args[1];
-   int ivalue = table->int_args[2];
-   Set_Int_Type *ip = Int_Things_To_Set;
-   char *svalue = table->string_args[2];
    Set_String_Type *sp = String_Things_To_Set;
-   int type = table->arg_type[2];
-   
-   if (type == STRING_TYPE)
+	
+   while (sp->what != NULL)
      {
-	while (sp->what != NULL)
+	if (!strcmp (sp->what, name))
 	  {
-	     if (!strcmp (sp->what, what))
-	       {
-		  char *ss;
+	     char *ss;
 		  
-		  if (sp->svaluep == NULL) return 0;
+	     if (sp->svaluep == NULL) return 0;
+	     
+	     ss = *sp->svaluep;
 		  
-		  ss = *sp->svaluep;
+	     slrn_free (ss);
+	     if (NULL == (ss = SLmake_string (value)))
+	       exit_malloc_error ();
 		  
-		  if (ss != NULL) SLFREE (ss);
-		  if (NULL == (ss = SLmake_string(svalue)))
-		    exit_malloc_error ();
-		  
-		  *sp->svaluep = ss;
-		  return 0;
-	       }
-	     sp++;
+	     *sp->svaluep = ss;
+	     return 0;
 	  }
+	sp++;
      }
-   else if (type == INT_TYPE) while (ip->what != NULL)
+   return -1;
+}
+
+/*}}}*/
+
+int slrn_set_integer_variable (char *name, int value) /*{{{*/
+{
+   Set_Int_Type *ip = Int_Things_To_Set;
+   
+   while (ip->what != NULL)
      {
-	if (!strcmp (ip->what, what))
+	if (!strcmp (ip->what, name))
 	  {
 	     if (ip->valuep == NULL) return 0;
-	     *ip->valuep = ivalue;
+	     *ip->valuep = value;
 	     return 0;
 	  }
 	ip++;
      }
-   exit_unknown_object ();
    return -1;
 }
 
+/*}}}*/
 
-typedef struct Server_List_Type
+int slrn_get_variable_value (char *name, int *type, char ***sval, int **ival) /*{{{*/
 {
-   struct Server_List_Type *next;
-   char *file;
-   char *host;
+   Set_String_Type *sp;
+   Set_Int_Type *ip;
+   
+   sp = String_Things_To_Set;
+   while (sp->what != NULL)
+     {
+	if (!strcmp (sp->what, name))
+	  {
+	     *sval = sp->svaluep;
+	     *type = STRING_TYPE;
+	     return 0;
+	  }
+	sp++;
+     }
+   
+   ip = Int_Things_To_Set;
+   while (ip->what != NULL)
+     {
+	if (!strcmp (ip->what, name))
+	  {
+	     *ival = ip->valuep;
+	     *type = INT_TYPE;
+	     return 0;
+	  }
+	ip++;
+     }
+   
+   return -1;
 }
-Server_List_Type;
 
-static Server_List_Type *Server_List;
+/*}}}*/
 
-static int server_fun (int argc, SLcmd_Cmd_Table_Type *table)
+static int set_variable_fun (int argc, SLcmd_Cmd_Table_Type *table) /*{{{*/
 {
-   char *file = NULL, *host;
-   char *the_file = table->string_args[2], *the_host = table->string_args[1];
-   int n;
-   Server_List_Type *s;
+   int ret;
+   char *what = table->string_args[1];
+   int ivalue = table->int_args[2];
+   char *svalue = table->string_args[2];
+   int type = table->arg_type[2];
    
-   n = strlen (the_file);
+   (void) argc;
    
-   s = (Server_List_Type *) safe_malloc (sizeof (Server_List_Type));
-   file = safe_malloc (n + 2 + strlen (the_host));
+   if (type == STRING_TYPE)
+     ret = slrn_set_string_variable (what, svalue);
+   else if (type == INT_TYPE) 
+     ret = slrn_set_integer_variable (what, ivalue);
+   else ret = -1;
    
-   strcpy (file, the_file);
-   host = file + (n + 1);
-   strcpy (host, the_host);
-   
-   s->next = Server_List;  Server_List = s;
-   s->host = host;
-   s->file = file;
+   if (ret != 0) exit_unknown_object ();
    return 0;
 }
 
-typedef struct
+/*}}}*/
+
+
+/*}}}*/
+
+/*{{{ Setting Color/Mono Attributes */
+
+typedef struct /*{{{*/
 {
    char *name;
    int value;
+   char *fg, *bg;
+   SLtt_Char_Type mono;
 }
+
+/*}}}*/
 Color_Handle_Type;
 
-Color_Handle_Type Color_Handles[] =
+/* default colors -- suitable for a color xterm */
+   
+static Color_Handle_Type Color_Handles[] = /*{{{*/
 {
-     {"normal", 0},
-     {"status", STATUS_COLOR},
-     {"menu", MENU_COLOR},
-     {"menu_press", MENU_PRESS_COLOR},
-     {"headers", HEADER_COLOR},
-     {"group", GROUP_COLOR},
-     {"subject", SUBJECT_COLOR},
-     {"author", AUTHOR_COLOR},
-     {"error", ERROR_COLOR},
-     {"cursor", CURSOR_COLOR},
-     {"article", ARTICLE_COLOR},
-     {"tree", TREE_COLOR},
-     {"quotes", QUOTE_COLOR},
-     {"signature", SIGNATURE_COLOR},
-     {"thread_number", THREAD_NUM_COLOR},
-     {"header_number", HEADER_NUMBER_COLOR},
-     {"high_score", HIGH_SCORE_COLOR},
-     {"description", GROUP_DESCR_COLOR},
-     {NULL, -1}
+     {"normal",		0,		"black",	"white", 0},
+     {"status",		STATUS_COLOR,	"yellow",	"blue", SLTT_REV_MASK},
+     {"menu",		MENU_COLOR,	"yellow",	"blue", SLTT_REV_MASK},
+     {"menu_press",	MENU_PRESS_COLOR,"blue",	"yellow", 0},
+     {"headers",	HEADER_COLOR,	"brightcyan",	"white", 0},
+     {"group",		GROUP_COLOR,	"blue",		"white", 0},
+     {"subject",	SUBJECT_COLOR,	"black",	"white", 0},
+     {"author",		AUTHOR_COLOR,	"magenta",	"white", 0},
+     {"error",		ERROR_COLOR,	"red",		"white", SLTT_BLINK_MASK},
+     {"cursor",		CURSOR_COLOR,	"brightgreen",	"white", SLTT_REV_MASK},
+     {"article",	ARTICLE_COLOR,	"blue",		"white", 0},
+     {"tree",		TREE_COLOR,	"red",		"white", 0},
+     {"quotes",		QUOTE_COLOR,	"red",		"white", 0},
+     {"signature",	SIGNATURE_COLOR,"red",		"white", 0},
+     {"thread_number",	THREAD_NUM_COLOR,"blue",	"white", SLTT_BOLD_MASK},
+     {"header_number",	HEADER_NUMBER_COLOR,"green",	"white", 0},
+     {"high_score",	HIGH_SCORE_COLOR,"red",		"white", SLTT_BOLD_MASK},
+     {"description",	GROUP_DESCR_COLOR,"magenta",	"white", 0},
+     {"grouplens_display",GROUPLENS_DISPLAY_COLOR,"blue","white", 0},
+     {"tilde",		SLRN_TILDE_COLOR,"green",	"white", SLTT_BOLD_MASK},
+     {"header_name",SLRN_HEADER_KEYWORD_COLOR,"green", "white", SLTT_BOLD_MASK},
+     {NULL, -1, NULL, NULL, 0}
 };
 
+/*}}}*/
 
-static int color_fun (int argc, SLcmd_Cmd_Table_Type *table)
+static int color_fun (int argc, SLcmd_Cmd_Table_Type *table) /*{{{*/
 {
    char *what = table->string_args[1];
    char *fg = table->string_args[2];
    char *bg = table->string_args[3];
-   
    Color_Handle_Type *ct = Color_Handles;
    
+   (void) argc;   
+   
    while (ct->name != NULL)
      {
 	if (!strcmp (ct->name, what))
@@ -501,18 +685,22 @@ static int color_fun (int argc, SLcmd_Cmd_Table_Type *table)
    return 0;
 }
 
+/*}}}*/
+
 #if SLANG_VERSION < 9932
 # ifdef __os2__
-void SLtt_set_mono (int a, char *b, SLtt_Char_Type c)
+void SLtt_set_mono (int a, char *b, SLtt_Char_Type c) /*{{{*/
 {
    (void) a;
    (void) b;
    (void) c;
 }
+
+/*}}}*/
 # endif
 #endif
 
-static int mono_fun (int argc, SLcmd_Cmd_Table_Type *table)
+static int mono_fun (int argc, SLcmd_Cmd_Table_Type *table) /*{{{*/
 {
    char *what = table->string_args[1];
    char *attr;
@@ -544,6 +732,11 @@ static int mono_fun (int argc, SLcmd_Cmd_Table_Type *table)
    return 0;
 }
 
+/*}}}*/
+
+/*}}}*/
+
+/*{{{ User Info Data (Hostname, etc) */
 
 /*----------------------------------------------------------------------*\
 * static int user_data_fun ();
@@ -559,33 +752,41 @@ static int mono_fun (int argc, SLcmd_Cmd_Table_Type *table)
  *   hostname		- full name of the current host
 \*----------------------------------------------------------------------*/
 
-typedef struct
+typedef struct /*{{{*/
 {
    char *name;
    char **addr;
    unsigned int size;
 }
+
+/*}}}*/
 User_Info_Variable_Type;
 
-User_Info_Variable_Type User_Info_Variables[] =
+static User_Info_Variable_Type User_Info_Variables[] = /*{{{*/
 {
-     {"realname", &Slrn_User_Info.realname, 0},
      {"username", &Slrn_User_Info.username, 0},
      {"hostname", (char **)Slrn_User_Info.host, MAX_HOST_NAME_LEN},
+     {"scorefile", &Slrn_Score_File, 0},
+   
+#define USER_INFO_OBSOLETE_STARTS_HERE 3
+   /* The following are obsolete and will not be supported in the future. */
+     {"realname", &Slrn_User_Info.realname, 0},
      {"replyto", &Slrn_User_Info.replyto, 0},
      {"organization", &Slrn_User_Info.org, 0},
      {"followup", &Slrn_User_Info.followup_string, 0},
      {"signature", &Slrn_User_Info.signature, 0},
-     {"scorefile", &Slrn_Score_File, 0},
      {"cc_followup_string", &Slrn_Courtesy_CC_Message, 0},
 #if SLRN_HAS_DECODE
      {"decode_directory", &Slrn_Decode_Directory, 0},
 #endif
      {"editor_command", &Slrn_Editor, 0},
+     {"quote_string", NULL, 0},
      {NULL, NULL, 0}
 };
 
-static int user_data_fun (int argc, SLcmd_Cmd_Table_Type *table)
+/*}}}*/
+
+static int user_data_fun (int argc, SLcmd_Cmd_Table_Type *table) /*{{{*/
 {
    char *what = table->string_args[0];
    char *field = table->string_args[1];
@@ -593,6 +794,8 @@ static int user_data_fun (int argc, SLcmd_Cmd_Table_Type *table)
    char **ptr, *contents;
    unsigned int n;
    
+   (void) argc;
+   
    while (u->name != NULL)
      {
 	if (!strcmp (u->name, what))
@@ -605,15 +808,18 @@ static int user_data_fun (int argc, SLcmd_Cmd_Table_Type *table)
 		  strncpy (contents, field, u->size);
 		  contents [u->size - 1] = 0;
 	       }
-	     else
+	     else if (NULL != (ptr = u->addr))
 	       {
-		  ptr = u->addr;
 		  contents = safe_malloc (n + 1);
 		  strcpy (contents, field);
 		  
-		  if (*ptr != NULL) SLFREE (*ptr);
+		  slrn_free (*ptr);
 		  *ptr = contents;
 	       }
+
+	     if (u >= User_Info_Variables + USER_INFO_OBSOLETE_STARTS_HERE)
+	       issue_obsolete_message ();
+	     
 	     return 0;
 	  }
 	u++;
@@ -622,6 +828,104 @@ static int user_data_fun (int argc, SLcmd_Cmd_Table_Type *table)
    return -1;
 }
 
+/*}}}*/
+
+/*}}}*/
+
+/*{{{ Server Newsrc Mapping Functions */
+
+typedef struct Server_List_Type /*{{{*/
+{
+   struct Server_List_Type *next;
+   char *file;
+   char *host;
+   char *username;
+   char *password;
+}
+
+/*}}}*/
+Server_List_Type;
+
+static Server_List_Type *Server_List;
+
+static Server_List_Type *find_server (char *host) /*{{{*/
+{
+   Server_List_Type *s = Server_List;
+   
+   while (s != NULL)
+     {
+	if (0 == slrn_case_strcmp ((unsigned char *)host, 
+				   (unsigned char *) s->host))
+	  break;
+	s = s->next;
+     }
+   return s;
+}
+
+/*}}}*/
+
+static int add_to_server_list (char *host, char *file, char *username, char *password) /*{{{*/
+{
+   Server_List_Type *s;
+   
+   if (NULL == (s = find_server (host)))
+     {
+	s = (Server_List_Type *) safe_malloc (sizeof (Server_List_Type));
+	memset ((char *) s, 0, sizeof (Server_List_Type));
+	s->next = Server_List;
+	Server_List = s;
+	s->host = slrn_safe_strmalloc (host);
+     }
+   
+   if (file != NULL)
+     {
+	slrn_free (s->file);
+	s->file = slrn_safe_strmalloc (file);
+     }
+
+   if (username != NULL)
+     {
+	slrn_free (s->username);
+	s->username = slrn_safe_strmalloc (username);
+     }
+
+   if (password != NULL)
+     {
+	slrn_free (s->password);
+	s->password = slrn_safe_strmalloc (password);
+     }
+   
+   return 0;
+}
+
+/*}}}*/
+
+static int server_fun (int argc, SLcmd_Cmd_Table_Type *table) /*{{{*/
+{
+   char *the_file = table->string_args[2], *the_host = table->string_args[1];
+      
+   (void) argc;
+   
+   return add_to_server_list (the_host, the_file, NULL, NULL);
+}
+
+/*}}}*/
+
+char *slrn_map_file_to_host (char *host) /*{{{*/
+{
+   Server_List_Type *s;
+   
+   if (NULL == (s = find_server (host)))
+     return NULL;
+   
+   return s->file;
+}
+
+/*}}}*/
+
+/*}}}*/
+
+/*{{{ Server Authorization Information */
 
 /*----------------------------------------------------------------------*\
 * static int nnrp_fun ();
@@ -636,101 +940,127 @@ static int user_data_fun (int argc, SLcmd_Cmd_Table_Type *table)
  *                         "host  username  password"
 \*----------------------------------------------------------------------*/
 
-static int nnrp_fun (int argc, SLcmd_Cmd_Table_Type *table)
+static int nnrp_fun (int argc, SLcmd_Cmd_Table_Type *table) /*{{{*/
 {
 #if SLRN_HAS_NNTP_SUPPORT
    char *server = table->string_args[1];
    char *name = table->string_args[2];
    char *pass = table->string_args[3];
-   unsigned int len;
-   
-   if (Slrn_NNTP_Server_Name == NULL) 
-     return 0;
-   
-   /* Skip this one it is not the one that we are looking for */
-   if (strcmp(server, Slrn_NNTP_Server_Name))
-     return 0;
-   
-   if (Slrn_User_Info.nnrpname != NULL) SLFREE (Slrn_User_Info.nnrpname);
-   if (Slrn_User_Info.nnrppass != NULL) SLFREE (Slrn_User_Info.nnrppass);
-   
-   len = strlen(name);
-   Slrn_User_Info.nnrpname = safe_malloc (len + 1);
-   strcpy(Slrn_User_Info.nnrpname, name);
-   
-   len = strlen(pass);
-   Slrn_User_Info.nnrppass = safe_malloc (len + 1);
-   strcpy(Slrn_User_Info.nnrppass, pass);
+
+   (void) argc;
+   return add_to_server_list (server, NULL, name, pass);
+#else
+   (void) argc;
+   (void) table;
+   return 0;
 #endif   
+}
+/*}}}*/
+
+int slrn_get_authorization (char *host, char **name, char **pass) /*{{{*/
+{
+   Server_List_Type *s;
+   char buf[256];
+   
+   *name = NULL;
+   *pass = NULL;
+   
+   s = find_server (host);
+
+   if (s == NULL)
+     return 0;
+   
+   if ((s->username == NULL) || (s->password == NULL))
+     return 0;
+   
+   if (*s->username == 0)
+     {
+	*buf = 0;
+	if (-1 == slrn_read_input ("Username", NULL, buf, 1))
+	  return -1;
+	SLFREE (s->username);
+	s->username = slrn_safe_strmalloc (buf);
+     }
+
+   if (*s->password == 0)
+     {
+	*buf = 0;
+	if (-1 == slrn_read_input_no_echo ("Password", NULL, buf, 1))
+	  return -1;
+	SLFREE (s->password);
+	s->password = slrn_safe_strmalloc (buf);
+     }
+
+   *name = s->username;
+   *pass = s->password;
+
    return 0;
 }
 
-static void slrn_init_keymaps (void)
+/*}}}*/
+
+/*}}}*/
+
+static void slrn_init_modes (void) /*{{{*/
 {
-   slrn_init_group_keymap ();
-   slrn_init_article_keymap ();
+   slrn_init_group_mode ();
+   slrn_init_article_mode ();
 }
 
-void slrn_startup_initialize (void)
+/*}}}*/
+
+void slrn_startup_initialize (void) /*{{{*/
 {
-   char *white = "white", *blue = "blue", *red = "red", *yellow = "yellow";
-   char *black = "black";
-   slrn_init_keymaps ();
+   Color_Handle_Type *h;
+   char *server;
+   
+   slrn_init_modes ();
    SLang_init_case_tables ();
    
    Slrn_Ignore_Quote_Regexp[0] = compile_quote_regexp ("^ ?[:>]");
    
-   /* default colors -- suitable for a color xterm */
-   
-   SLtt_set_color (0, NULL, black, white);
-   SLtt_set_color (STATUS_COLOR, NULL, yellow, blue);
-   SLtt_set_mono (STATUS_COLOR, NULL, SLTT_REV_MASK);
-   SLtt_set_color (MENU_COLOR, NULL, yellow, blue);
-   SLtt_set_mono (MENU_COLOR, NULL, SLTT_REV_MASK);
-   SLtt_set_color (HEADER_COLOR, NULL, "brightcyan", white);
-   SLtt_set_mono (HEADER_COLOR, NULL, SLTT_BOLD_MASK);
-   SLtt_set_color (GROUP_COLOR, NULL, blue, white);
-   SLtt_set_mono (GROUP_COLOR, NULL, 0);
-   SLtt_set_color (SUBJECT_COLOR, NULL, black, white);
-   SLtt_set_mono (SUBJECT_COLOR, NULL, 0);
-   SLtt_set_color (AUTHOR_COLOR, NULL, "magenta", white);
-   SLtt_set_mono (AUTHOR_COLOR, NULL, 0);
-   SLtt_set_color (ERROR_COLOR, NULL, red, white);
-   SLtt_set_mono (ERROR_COLOR, NULL, SLTT_BLINK_MASK);
-   SLtt_set_color (CURSOR_COLOR, NULL, "brightgreen", white);
-   SLtt_set_mono (CURSOR_COLOR, NULL, SLTT_REV_MASK);
-   SLtt_set_color (ARTICLE_COLOR, NULL, blue, white);
-   SLtt_set_mono (ARTICLE_COLOR, NULL, 0);
-   SLtt_set_color (TREE_COLOR, NULL, red, white);
-   SLtt_set_mono (TREE_COLOR, NULL, 0);
-   SLtt_set_color (QUOTE_COLOR, NULL, red, white);
-   SLtt_set_mono (QUOTE_COLOR, NULL, 0);
-   SLtt_set_color (SIGNATURE_COLOR, NULL, red, white);
-   SLtt_set_mono (SIGNATURE_COLOR, NULL, 0);
-   SLtt_set_color (THREAD_NUM_COLOR, NULL, blue, white);
-   SLtt_set_mono (THREAD_NUM_COLOR, NULL, SLTT_BOLD_MASK);
-   SLtt_set_color (HIGH_SCORE_COLOR, NULL, red, white);
-   SLtt_set_mono (HIGH_SCORE_COLOR, NULL, SLTT_BOLD_MASK);
-   SLtt_set_color (HEADER_NUMBER_COLOR, NULL, "green", white);
-   SLtt_set_mono (HEADER_NUMBER_COLOR, NULL, 0);
-   SLtt_set_color (MENU_PRESS_COLOR, NULL, blue, yellow);
-   SLtt_set_mono (MENU_PRESS_COLOR, NULL, 0);
-   SLtt_set_color (GROUP_DESCR_COLOR, NULL, "magenta", white);
-   SLtt_set_mono (GROUP_DESCR_COLOR, NULL, 0);
-   SLtt_set_mono (HIGH_SCORE_COLOR, NULL, SLTT_BOLD_MASK);
-   
-   
+   h = Color_Handles;
+   while (h->name != NULL)
+     {
+	SLtt_set_color (h->value, NULL, h->fg, h->bg);
+	SLtt_set_mono (h->value, NULL, h->mono);
+	h++;
+     }
+
 #ifndef __os2__
    /* We are not using the blink characters (unless in mono) */
    SLtt_Blink_Mode = 0;
 #endif
+   
+   switch (Slrn_Server_Id)
+     {
+      case SLRN_SERVER_ID_NNTP:
+	server = "NNTP";
+	break;
+	
+      case SLRN_SERVER_ID_SPOOL:
+	server = "SPOOL";
+	break;
+	
+      default:
+	server = NULL;
+     }
+   
+   if ((server != NULL) 
+       && (0 == SLdefine_for_ifdef (server)))
+     {
+	slrn_exit_error ("Unable to add preprocessor name %s.", server);
+     }
 }
 
-void slrn_read_startup_file (char *name)
+
+/*}}}*/
+
+void slrn_read_startup_file (char *name) /*{{{*/
 {
    FILE *fp;
-   char file[256];
-   char line[256];
+   char file [SLRN_MAX_PATH_LEN];
+   char line [256];
    SLPreprocess_Type pt;
 
    if (-1 == slrn_init_readline ())
@@ -748,6 +1078,8 @@ void slrn_read_startup_file (char *name)
    fp = slrn_open_home_file (name, "r", file, 0);
    if (fp == NULL) return;
    
+   slrn_message ("Reading startup file %s.", file);
+
    This_File = file;
    This_Line = line;
    
@@ -767,18 +1099,21 @@ void slrn_read_startup_file (char *name)
    SLprep_close_prep (&pt);
 }
 
+/*}}}*/
 
-
-char *slrn_map_file_to_host (char *host)
+static int interpret_fun (int argc, SLcmd_Cmd_Table_Type *table) /*{{{*/
 {
-   Server_List_Type *s = Server_List;
+#if SLRN_HAS_SLANG
+   char *file = table->string_args [1];
    
-   while (s != NULL)
-     {
-	if (!strcmp (host, s->host)) return s->file;
-	s = s->next;
-     }
-   return NULL;
+   (void) argc;
+   if (Slrn_Use_Slang == 0) return 0;
+   return slrn_eval_slang_file (file);
+#else
+   (void) argc;
+   return 0;
+#endif
 }
 
+/*}}}*/
 
