@@ -3,14 +3,14 @@
  *  All Rights Reserved.
  */
 #include "config.h"
-#include "features.h"
+#include "slrnfeat.h"
 
 #include <stdio.h>
 #include <ssdef.h>
 #include <string.h>
 
 
-/* #include "buffer.h" */
+#include "vms.h"
 #include "vmsmail.h"
 
 extern int mail$send_add_bodypart ();
